"""
Support for Yamaha MusicCast Receivers.

For more details about this platform, please refer to the documentation at
https://www.home-assistant.io/components/media_player.yamaha_musiccast/
"""
import logging

import voluptuous as vol

from homeassistant.components.media_player import (
    MEDIA_TYPE_MUSIC, PLATFORM_SCHEMA, SUPPORT_NEXT_TRACK, SUPPORT_PAUSE,
    SUPPORT_PLAY, SUPPORT_PREVIOUS_TRACK, SUPPORT_SELECT_SOURCE, SUPPORT_STOP,
    SUPPORT_TURN_OFF, SUPPORT_TURN_ON, SUPPORT_VOLUME_MUTE, SUPPORT_VOLUME_SET,
    MediaPlayerDevice)
from homeassistant.const import (
    CONF_HOST, CONF_PORT, STATE_IDLE, STATE_ON, STATE_PAUSED, STATE_PLAYING,
    STATE_UNKNOWN)
import homeassistant.helpers.config_validation as cv
import homeassistant.util.dt as dt_util

REQUIREMENTS = ['pymusiccast==0.1.6']

_LOGGER = logging.getLogger(__name__)

SUPPORTED_FEATURES = (
    SUPPORT_PLAY | SUPPORT_PAUSE | SUPPORT_STOP |
    SUPPORT_PREVIOUS_TRACK | SUPPORT_NEXT_TRACK |
    SUPPORT_TURN_ON | SUPPORT_TURN_OFF |
    SUPPORT_VOLUME_SET | SUPPORT_VOLUME_MUTE |
    SUPPORT_SELECT_SOURCE
)

KNOWN_HOSTS_KEY = 'data_yamaha_musiccast'
INTERVAL_SECONDS = 'interval_seconds'

DEFAULT_PORT = 5005
DEFAULT_INTERVAL = 480

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Required(CONF_HOST): cv.string,
    vol.Optional(CONF_PORT, default=DEFAULT_PORT): cv.positive_int,
    vol.Optional(INTERVAL_SECONDS, default=DEFAULT_INTERVAL): cv.positive_int,
})


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up the Yamaha MusicCast platform."""
    import socket
    import pymusiccast

    known_hosts = hass.data.get(KNOWN_HOSTS_KEY)
    if known_hosts is None:
        known_hosts = hass.data[KNOWN_HOSTS_KEY] = []
    _LOGGER.debug("known_hosts: %s", known_hosts)

    host = config.get(CONF_HOST)
    port = config.get(CONF_PORT)
    interval = config.get(INTERVAL_SECONDS)

    # Get IP of host to prevent duplicates
    try:
        ipaddr = socket.gethostbyname(host)
    except (OSError) as error:
        _LOGGER.error(
            "Could not communicate with %s:%d: %s", host, port, error)
        return

    if [item for item in known_hosts if item[0] == ipaddr]:
        _LOGGER.warning("Host %s:%d already registered", host, port)
        return

    if [item for item in known_hosts if item[1] == port]:
        _LOGGER.warning("Port %s:%d already registered", host, port)
        return

    reg_host = (ipaddr, port)
    known_hosts.append(reg_host)

    try:
        receiver = pymusiccast.McDevice(
            ipaddr, udp_port=port, mc_interval=interval)
    except pymusiccast.exceptions.YMCInitError as err:
        _LOGGER.error(err)
        receiver = None

    if receiver:
        for zone in receiver.zones:
            _LOGGER.debug(
                "Receiver: %s / Port: %d / Zone: %s", receiver, port, zone)
            add_entities(
                [YamahaDevice(receiver, receiver.zones[zone])], True)
    else:
        known_hosts.remove(reg_host)


class YamahaDevice(MediaPlayerDevice):
    """Representation of a Yamaha MusicCast device."""

    def __init__(self, recv, zone):
        """Initialize the Yamaha MusicCast device."""
        self._recv = recv
        self._name = recv.name
        self._source = None
        self._source_list = []
        self._zone = zone
        self.mute = False
        self.media_status = None
        self.media_status_received = None
        self.power = STATE_UNKNOWN
        self.status = STATE_UNKNOWN
        self.volume = 0
        self.volume_max = 0
        self._recv.set_yamaha_device(self)
        self._zone.set_yamaha_device(self)

    @property
    def name(self):
        """Return the name of the device."""
        return "{} ({})".format(self._name, self._zone.zone_id)

    @property
    def state(self):
        """Return the state of the device."""
        if self.power == STATE_ON and self.status is not STATE_UNKNOWN:
            return self.status
        return self.power

    @property
    def should_poll(self):
        """Push an update after each command."""
        return True

    @property
    def is_volume_muted(self):
        """Boolean if volume is currently muted."""
        return self.mute

    @property
    def volume_level(self):
        """Volume level of the media player (0..1)."""
        return self.volume

    @property
    def supported_features(self):
        """Flag of features that are supported."""
        return SUPPORTED_FEATURES

    @property
    def source(self):
        """Return the current input source."""
        return self._source

    @property
    def source_list(self):
        """List of available input sources."""
        return self._source_list

    @source_list.setter
    def source_list(self, value):
        """Set source_list attribute."""
        self._source_list = value

    @property
    def media_content_type(self):
        """Return the media content type."""
        return MEDIA_TYPE_MUSIC

    @property
    def media_duration(self):
        """Duration of current playing media in seconds."""
        return self.media_status.media_duration \
            if self.media_status else None

    @property
    def media_image_url(self):
        """Image url of current playing media."""
        return self.media_status.media_image_url \
            if self.media_status else None

    @property
    def media_artist(self):
        """Artist of current playing media, music track only."""
        return self.media_status.media_artist if self.media_status else None

    @property
    def media_album(self):
        """Album of current playing media, music track only."""
        return self.media_status.media_album if self.media_status else None

    @property
    def media_track(self):
        """Track number of current playing media, music track only."""
        return self.media_status.media_track if self.media_status else None

    @property
    def media_title(self):
        """Title of current playing media."""
        return self.media_status.media_title if self.media_status else None

    @property
    def media_position(self):
        """Position of current playing media in seconds."""
        if self.media_status and self.state in \
                [STATE_PLAYING, STATE_PAUSED, STATE_IDLE]:
            return self.media_status.media_position

    @property
    def media_position_updated_at(self):
        """When was the position of the current playing media valid.

        Returns value from homeassistant.util.dt.utcnow().
        """
        return self.media_status_received if self.media_status else None

    def update(self):
        """Get the latest details from the device."""
        _LOGGER.debug("update: %s", self.entity_id)
        self._recv.update_status()
        self._zone.update_status()

    def update_hass(self):
        """Push updates to HASS."""
        if self.entity_id:
            _LOGGER.debug("update_hass: pushing updates")
            self.schedule_update_ha_state()
            return True

    def turn_on(self):
        """Turn on specified media player or all."""
        _LOGGER.debug("Turn device: on")
        self._zone.set_power(True)

    def turn_off(self):
        """Turn off specified media player or all."""
        _LOGGER.debug("Turn device: off")
        self._zone.set_power(False)

    def media_play(self):
        """Send the media player the command for play/pause."""
        _LOGGER.debug("Play")
        self._recv.set_playback("play")

    def media_pause(self):
        """Send the media player the command for pause."""
        _LOGGER.debug("Pause")
        self._recv.set_playback("pause")

    def media_stop(self):
        """Send the media player the stop command."""
        _LOGGER.debug("Stop")
        self._recv.set_playback("stop")

    def media_previous_track(self):
        """Send the media player the command for prev track."""
        _LOGGER.debug("Previous")
        self._recv.set_playback("previous")

    def media_next_track(self):
        """Send the media player the command for next track."""
        _LOGGER.debug("Next")
        self._recv.set_playback("next")

    def mute_volume(self, mute):
        """Send mute command."""
        _LOGGER.debug("Mute volume: %s", mute)
        self._zone.set_mute(mute)

    def set_volume_level(self, volume):
        """Set volume level, range 0..1."""
        _LOGGER.debug("Volume level: %.2f / %d",
                      volume, volume * self.volume_max)
        self._zone.set_volume(volume * self.volume_max)

    def select_source(self, source):
        """Send the media player the command to select input source."""
        _LOGGER.debug("select_source: %s", source)
        self.status = STATE_UNKNOWN
        self._zone.set_input(source)

    def new_media_status(self, status):
        """Handle updates of the media status."""
        _LOGGER.debug("new media_status arrived")
        self.media_status = status
        self.media_status_received = dt_util.utcnow()



"""
Connect two Home Assistant instances via MQTT.

For more details about this component, please refer to the documentation at
https://home-assistant.io/components/mqtt_eventstream/
"""
import asyncio
import json

import voluptuous as vol

from homeassistant.core import callback
from homeassistant.components.mqtt import (
    valid_publish_topic, valid_subscribe_topic)
from homeassistant.const import (
    ATTR_SERVICE_DATA, EVENT_CALL_SERVICE, EVENT_SERVICE_EXECUTED,
    EVENT_STATE_CHANGED, EVENT_TIME_CHANGED, MATCH_ALL)
from homeassistant.core import EventOrigin, State
import homeassistant.helpers.config_validation as cv
from homeassistant.helpers.json import JSONEncoder

DOMAIN = 'mqtt_eventstream'
DEPENDENCIES = ['mqtt']

CONF_PUBLISH_TOPIC = 'publish_topic'
CONF_SUBSCRIBE_TOPIC = 'subscribe_topic'
CONF_PUBLISH_EVENTSTREAM_RECEIVED = 'publish_eventstream_received'
CONF_IGNORE_EVENT = 'ignore_event'

CONFIG_SCHEMA = vol.Schema({
    DOMAIN: vol.Schema({
        vol.Optional(CONF_PUBLISH_TOPIC): valid_publish_topic,
        vol.Optional(CONF_SUBSCRIBE_TOPIC): valid_subscribe_topic,
        vol.Optional(CONF_PUBLISH_EVENTSTREAM_RECEIVED, default=False):
            cv.boolean,
        vol.Optional(CONF_IGNORE_EVENT, default=[]): cv.ensure_list
    }),
}, extra=vol.ALLOW_EXTRA)


@asyncio.coroutine
def async_setup(hass, config):
    """Set up the MQTT eventstream component."""
    mqtt = hass.components.mqtt
    conf = config.get(DOMAIN, {})
    pub_topic = conf.get(CONF_PUBLISH_TOPIC)
    sub_topic = conf.get(CONF_SUBSCRIBE_TOPIC)
    ignore_event = conf.get(CONF_IGNORE_EVENT)

    @callback
    def _event_publisher(event):
        """Handle events by publishing them on the MQTT queue."""
        if event.origin != EventOrigin.local:
            return
        if event.event_type == EVENT_TIME_CHANGED:
            return

        # User-defined events to ignore
        if event.event_type in ignore_event:
            return

        # Filter out the events that were triggered by publishing
        # to the MQTT topic, or you will end up in an infinite loop.
        if event.event_type == EVENT_CALL_SERVICE:
            if (
                    event.data.get('domain') == mqtt.DOMAIN and
                    event.data.get('service') == mqtt.SERVICE_PUBLISH and
                    event.data[ATTR_SERVICE_DATA].get('topic') == pub_topic
            ):
                return

        # Filter out all the "event service executed" events because they
        # are only used internally by core as callbacks for blocking
        # during the interval while a service is being executed.
        # They will serve no purpose to the external system,
        # and thus are unnecessary traffic.
        # And at any rate it would cause an infinite loop to publish them
        # because publishing to an MQTT topic itself triggers one.
        if event.event_type == EVENT_SERVICE_EXECUTED:
            return

        event_info = {'event_type': event.event_type, 'event_data': event.data}
        msg = json.dumps(event_info, cls=JSONEncoder)
        mqtt.async_publish(pub_topic, msg)

    # Only listen for local events if you are going to publish them.
    if pub_topic:
        hass.bus.async_listen(MATCH_ALL, _event_publisher)

    # Process events from a remote server that are received on a queue.
    @callback
    def _event_receiver(topic, payload, qos):
        """Receive events published by and fire them on this hass instance."""
        event = json.loads(payload)
        event_type = event.get('event_type')
        event_data = event.get('event_data')

        # Special case handling for event STATE_CHANGED
        # We will try to convert state dicts back to State objects
        # Copied over from the _handle_api_post_events_event method
        # of the api component.
        if event_type == EVENT_STATE_CHANGED and event_data:
            for key in ('old_state', 'new_state'):
                state = State.from_dict(event_data.get(key))

                if state:
                    event_data[key] = state

        hass.bus.async_fire(
            event_type,
            event_data=event_data,
            origin=EventOrigin.remote
        )

    # Only subscribe if you specified a topic.
    if sub_topic:
        yield from mqtt.async_subscribe(sub_topic, _event_receiver)

    return True



"""
Support Legacy API password auth provider.

It will be removed when auth system production ready
"""
import hmac
from typing import Any, Dict, Optional, cast

import voluptuous as vol

from homeassistant.components.http import HomeAssistantHTTP  # noqa: F401
from homeassistant.core import callback
from homeassistant.exceptions import HomeAssistantError

from . import AuthProvider, AUTH_PROVIDER_SCHEMA, AUTH_PROVIDERS, LoginFlow
from ..models import Credentials, UserMeta


USER_SCHEMA = vol.Schema({
    vol.Required('username'): str,
})


CONFIG_SCHEMA = AUTH_PROVIDER_SCHEMA.extend({
}, extra=vol.PREVENT_EXTRA)

LEGACY_USER_NAME = 'Legacy API password user'


class InvalidAuthError(HomeAssistantError):
    """Raised when submitting invalid authentication."""


@AUTH_PROVIDERS.register('legacy_api_password')
class LegacyApiPasswordAuthProvider(AuthProvider):
    """Example auth provider based on hardcoded usernames and passwords."""

    DEFAULT_TITLE = 'Legacy API Password'

    async def async_login_flow(self, context: Optional[Dict]) -> LoginFlow:
        """Return a flow to login."""
        return LegacyLoginFlow(self)

    @callback
    def async_validate_login(self, password: str) -> None:
        """Validate a username and password."""
        hass_http = getattr(self.hass, 'http', None)  # type: HomeAssistantHTTP

        if not hmac.compare_digest(hass_http.api_password.encode('utf-8'),
                                   password.encode('utf-8')):
            raise InvalidAuthError

    async def async_get_or_create_credentials(
            self, flow_result: Dict[str, str]) -> Credentials:
        """Return credentials for this login."""
        credentials = await self.async_credentials()
        if credentials:
            return credentials[0]

        return self.async_create_credentials({})

    async def async_user_meta_for_credentials(
            self, credentials: Credentials) -> UserMeta:
        """
        Return info for the user.

        Will be used to populate info when creating a new user.
        """
        return UserMeta(name=LEGACY_USER_NAME, is_active=True)


class LegacyLoginFlow(LoginFlow):
    """Handler for the login flow."""

    async def async_step_init(
            self, user_input: Optional[Dict[str, str]] = None) \
            -> Dict[str, Any]:
        """Handle the step of the form."""
        errors = {}

        hass_http = getattr(self.hass, 'http', None)
        if hass_http is None or not hass_http.api_password:
            return self.async_abort(
                reason='no_api_password_set'
            )

        if user_input is not None:
            try:
                cast(LegacyApiPasswordAuthProvider, self._auth_provider)\
                    .async_validate_login(user_input['password'])
            except InvalidAuthError:
                errors['base'] = 'invalid_auth'

            if not errors:
                return await self.async_finish({})

        return self.async_show_form(
            step_id='init',
            data_schema=vol.Schema({'password': str}),
            errors=errors,
        )



"""
Support for Yeelight Sunflower color bulbs (not Yeelight Blue or WiFi).

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/light.yeelightsunflower/
"""
import logging

import voluptuous as vol

import homeassistant.helpers.config_validation as cv
from homeassistant.components.light import (
    Light, ATTR_HS_COLOR, SUPPORT_COLOR, ATTR_BRIGHTNESS,
    SUPPORT_BRIGHTNESS, PLATFORM_SCHEMA)
from homeassistant.const import CONF_HOST
import homeassistant.util.color as color_util

REQUIREMENTS = ['yeelightsunflower==0.0.10']

_LOGGER = logging.getLogger(__name__)

SUPPORT_YEELIGHT_SUNFLOWER = (SUPPORT_BRIGHTNESS | SUPPORT_COLOR)

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Required(CONF_HOST): cv.string
})


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up the Yeelight Sunflower Light platform."""
    import yeelightsunflower

    host = config.get(CONF_HOST)
    hub = yeelightsunflower.Hub(host)

    if not hub.available:
        _LOGGER.error("Could not connect to Yeelight Sunflower hub")
        return False

    add_entities(SunflowerBulb(light) for light in hub.get_lights())


class SunflowerBulb(Light):
    """Representation of a Yeelight Sunflower Light."""

    def __init__(self, light):
        """Initialize a Yeelight Sunflower bulb."""
        self._light = light
        self._available = light.available
        self._brightness = light.brightness
        self._is_on = light.is_on
        self._hs_color = light.rgb_color

    @property
    def name(self):
        """Return the display name of this light."""
        return 'sunflower_{}'.format(self._light.zid)

    @property
    def available(self):
        """Return True if entity is available."""
        return self._available

    @property
    def is_on(self):
        """Return true if light is on."""
        return self._is_on

    @property
    def brightness(self):
        """Return the brightness is 0-255; Yeelight's brightness is 0-100."""
        return int(self._brightness / 100 * 255)

    @property
    def hs_color(self):
        """Return the color property."""
        return self._hs_color

    @property
    def supported_features(self):
        """Flag supported features."""
        return SUPPORT_YEELIGHT_SUNFLOWER

    def turn_on(self, **kwargs):
        """Instruct the light to turn on, optionally set colour/brightness."""
        # when no arguments, just turn light on (full brightness)
        if not kwargs:
            self._light.turn_on()
        else:
            if ATTR_HS_COLOR in kwargs and ATTR_BRIGHTNESS in kwargs:
                rgb = color_util.color_hs_to_RGB(*kwargs[ATTR_HS_COLOR])
                bright = int(kwargs[ATTR_BRIGHTNESS] / 255 * 100)
                self._light.set_all(rgb[0], rgb[1], rgb[2], bright)
            elif ATTR_HS_COLOR in kwargs:
                rgb = color_util.color_hs_to_RGB(*kwargs[ATTR_HS_COLOR])
                self._light.set_rgb_color(rgb[0], rgb[1], rgb[2])
            elif ATTR_BRIGHTNESS in kwargs:
                bright = int(kwargs[ATTR_BRIGHTNESS] / 255 * 100)
                self._light.set_brightness(bright)

    def turn_off(self, **kwargs):
        """Instruct the light to turn off."""
        self._light.turn_off()

    def update(self):
        """Fetch new state data for this light and update local values."""
        self._light.update()
        self._available = self._light.available
        self._brightness = self._light.brightness
        self._is_on = self._light.is_on
        self._hs_color = color_util.color_RGB_to_hs(*self._light.rgb_color)



"""
Support for functionality to cache some data for AI-Speaker.
"""
import socket
import logging

# LV settings
G_EMPTY_OPTION = "-"

# TTS settings
GLOBAL_TTS_RATE = 1
GLOBAL_TTS_PITCH = 1
GLOBAL_TTS_VOICE = 'pl-pl-x-oda-local'

# audio nature
G_AN_RADIO = "Radio"
G_AN_PODCAST = "Podcast"
G_AN_MUSIC = "Music"
G_AN_AUDIOBOOK = "AudioBook"
G_AN_NEWS = "News"
G_AN_LOCAL = "Local"


# tokens
G_OFFLINE_MODE = False


GLOBAL_MY_IP = None
GLOBAL_MY_SSID = None
_LOGGER = logging.getLogger(__name__)

# devices fully supported by ais dom
G_MODEL_SONOFF_S20 = "s20"
G_MODEL_SONOFF_SLAMPHER = "slampher"
G_MODEL_SONOFF_TOUCH = "sonoff_touch"
G_MODEL_SONOFF_TH = "sonoff_th"
G_MODEL_SONOFF_B1 = "sonoff_b1"
G_MODEL_SONOFF_POW = "sonoff_pow"
G_MODEL_SONOFF_DUAL = "sonoff_dual"
G_MODEL_SONOFF_BASIC = "sonoff_basic"
G_MODEL_SONOFF_IFAN = "sonoff_ifan"
#

def set_my_ssid(ssid):
    global GLOBAL_MY_SSID
    GLOBAL_MY_SSID = ssid

def get_my_global_ip():
    if GLOBAL_MY_IP is None:
        set_global_my_ip()
    return GLOBAL_MY_IP


def set_global_my_ip():
    global GLOBAL_MY_IP
    try:
        GLOBAL_MY_IP = (([ip for ip in socket.gethostbyname_ex(socket.gethostname())[2]
                  if not ip.startswith('127.')] or [[(s.connect(('8.8.8.8', 53)),
                  s.getsockname()[0], s.close()) for s in
                  [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]][0][1]])
                  + ['no IP found'])[0]
    except Exception as e:
        _LOGGER.error("Error: " + str(e))
        GLOBAL_MY_IP = '127.0.0.1'


set_global_my_ip()




"""
Support gathering system information of hosts which are running netdata.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/sensor.netdata/
"""
from datetime import timedelta
import logging

import voluptuous as vol

from homeassistant.components.sensor import PLATFORM_SCHEMA
from homeassistant.const import (
    CONF_HOST, CONF_ICON, CONF_NAME, CONF_PORT, CONF_RESOURCES)
from homeassistant.exceptions import PlatformNotReady
from homeassistant.helpers.aiohttp_client import async_get_clientsession
import homeassistant.helpers.config_validation as cv
from homeassistant.helpers.entity import Entity
from homeassistant.util import Throttle

REQUIREMENTS = ['netdata==0.1.2']

_LOGGER = logging.getLogger(__name__)

MIN_TIME_BETWEEN_UPDATES = timedelta(minutes=1)

CONF_ELEMENT = 'element'

DEFAULT_HOST = 'localhost'
DEFAULT_NAME = 'Netdata'
DEFAULT_PORT = 19999

DEFAULT_ICON = 'mdi:desktop-classic'

RESOURCE_SCHEMA = vol.Any({
    vol.Required(CONF_ELEMENT): cv.string,
    vol.Optional(CONF_ICON, default=DEFAULT_ICON): cv.icon,
    vol.Optional(CONF_NAME): cv.string,
})

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Optional(CONF_HOST, default=DEFAULT_HOST): cv.string,
    vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
    vol.Optional(CONF_PORT, default=DEFAULT_PORT): cv.port,
    vol.Required(CONF_RESOURCES): vol.Schema({cv.string: RESOURCE_SCHEMA}),
})


async def async_setup_platform(
        hass, config, async_add_entities, discovery_info=None):
    """Set up the Netdata sensor."""
    from netdata import Netdata

    name = config.get(CONF_NAME)
    host = config.get(CONF_HOST)
    port = config.get(CONF_PORT)
    resources = config.get(CONF_RESOURCES)

    session = async_get_clientsession(hass)
    netdata = NetdataData(Netdata(host, hass.loop, session, port=port))
    await netdata.async_update()

    if netdata.api.metrics is None:
        raise PlatformNotReady

    dev = []
    for entry, data in resources.items():
        sensor = entry
        element = data[CONF_ELEMENT]
        sensor_name = icon = None
        try:
            resource_data = netdata.api.metrics[sensor]
            unit = '%' if resource_data['units'] == 'percentage' else \
                resource_data['units']
            if data is not None:
                sensor_name = data.get(CONF_NAME)
                icon = data.get(CONF_ICON)
        except KeyError:
            _LOGGER.error("Sensor is not available: %s", sensor)
            continue

        dev.append(NetdataSensor(
            netdata, name, sensor, sensor_name, element, icon, unit))

    async_add_entities(dev, True)


class NetdataSensor(Entity):
    """Implementation of a Netdata sensor."""

    def __init__(
            self, netdata, name, sensor, sensor_name, element, icon, unit):
        """Initialize the Netdata sensor."""
        self.netdata = netdata
        self._state = None
        self._sensor = sensor
        self._element = element
        self._sensor_name = self._sensor if sensor_name is None else \
            sensor_name
        self._name = name
        self._icon = icon
        self._unit_of_measurement = unit

    @property
    def name(self):
        """Return the name of the sensor."""
        return '{} {}'.format(self._name, self._sensor_name)

    @property
    def unit_of_measurement(self):
        """Return the unit the value is expressed in."""
        return self._unit_of_measurement

    @property
    def icon(self):
        """Return the icon to use in the frontend, if any."""
        return self._icon

    @property
    def state(self):
        """Return the state of the resources."""
        return self._state

    @property
    def available(self):
        """Could the resource be accessed during the last update call."""
        return self.netdata.available

    async def async_update(self):
        """Get the latest data from Netdata REST API."""
        await self.netdata.async_update()
        resource_data = self.netdata.api.metrics.get(self._sensor)
        self._state = round(
            resource_data['dimensions'][self._element]['value'], 2)


class NetdataData:
    """The class for handling the data retrieval."""

    def __init__(self, api):
        """Initialize the data object."""
        self.api = api
        self.available = True

    @Throttle(MIN_TIME_BETWEEN_UPDATES)
    async def async_update(self):
        """Get the latest data from the Netdata REST API."""
        from netdata.exceptions import NetdataError

        try:
            await self.api.get_allmetrics()
            self.available = True
        except NetdataError:
            _LOGGER.error("Unable to retrieve data from Netdata")
            self.available = False



"""
Interfaces with Verisure sensors.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/binary_sensor.verisure/
"""
import logging

from homeassistant.components.binary_sensor import BinarySensorDevice
from homeassistant.components.verisure import CONF_DOOR_WINDOW
from homeassistant.components.verisure import HUB as hub

_LOGGER = logging.getLogger(__name__)


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up the Verisure binary sensors."""
    sensors = []
    hub.update_overview()

    if int(hub.config.get(CONF_DOOR_WINDOW, 1)):
        sensors.extend([
            VerisureDoorWindowSensor(device_label)
            for device_label in hub.get(
                "$.doorWindow.doorWindowDevice[*].deviceLabel")])
    add_entities(sensors)


class VerisureDoorWindowSensor(BinarySensorDevice):
    """Representation of a Verisure door window sensor."""

    def __init__(self, device_label):
        """Initialize the Verisure door window sensor."""
        self._device_label = device_label

    @property
    def name(self):
        """Return the name of the binary sensor."""
        return hub.get_first(
            "$.doorWindow.doorWindowDevice[?(@.deviceLabel=='%s')].area",
            self._device_label)

    @property
    def is_on(self):
        """Return the state of the sensor."""
        return hub.get_first(
            "$.doorWindow.doorWindowDevice[?(@.deviceLabel=='%s')].state",
            self._device_label) == "OPEN"

    @property
    def available(self):
        """Return True if entity is available."""
        return hub.get_first(
            "$.doorWindow.doorWindowDevice[?(@.deviceLabel=='%s')]",
            self._device_label) is not None

    # pylint: disable=no-self-use
    def update(self):
        """Update the state of the sensor."""
        hub.update_overview()



"""
Support for Tahoma sensors.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/sensor.tahoma/
"""

import logging
from datetime import timedelta

from homeassistant.helpers.entity import Entity
from homeassistant.components.tahoma import (
    DOMAIN as TAHOMA_DOMAIN, TahomaDevice)
from homeassistant.const import ATTR_BATTERY_LEVEL

DEPENDENCIES = ['tahoma']

_LOGGER = logging.getLogger(__name__)

SCAN_INTERVAL = timedelta(seconds=60)

ATTR_RSSI_LEVEL = 'rssi_level'


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up Tahoma controller devices."""
    controller = hass.data[TAHOMA_DOMAIN]['controller']
    devices = []
    for device in hass.data[TAHOMA_DOMAIN]['devices']['sensor']:
        devices.append(TahomaSensor(device, controller))
    add_entities(devices, True)


class TahomaSensor(TahomaDevice, Entity):
    """Representation of a Tahoma Sensor."""

    def __init__(self, tahoma_device, controller):
        """Initialize the sensor."""
        self.current_value = None
        self._available = False
        super().__init__(tahoma_device, controller)

    @property
    def state(self):
        """Return the name of the sensor."""
        return self.current_value

    @property
    def unit_of_measurement(self):
        """Return the unit of measurement of this entity, if any."""
        if self.tahoma_device.type == 'Temperature Sensor':
            return None
        if self.tahoma_device.type == 'io:SomfyContactIOSystemSensor':
            return None
        if self.tahoma_device.type == 'io:LightIOSystemSensor':
            return 'lx'
        if self.tahoma_device.type == 'Humidity Sensor':
            return '%'

    def update(self):
        """Update the state."""
        self.controller.get_states([self.tahoma_device])
        if self.tahoma_device.type == 'io:LightIOSystemSensor':
            self.current_value = self.tahoma_device.active_states[
                'core:LuminanceState']
        if self.tahoma_device.type == 'io:SomfyContactIOSystemSensor':
            self.current_value = self.tahoma_device.active_states[
                'core:ContactState']

        self._available = bool(self.tahoma_device.active_states.get(
            'core:StatusState') == 'available')

        _LOGGER.debug("Update %s, value: %d", self._name, self.current_value)

    @property
    def device_state_attributes(self):
        """Return the device state attributes."""
        attr = {}
        super_attr = super().device_state_attributes
        if super_attr is not None:
            attr.update(super_attr)

        if 'core:RSSILevelState' in self.tahoma_device.active_states:
            attr[ATTR_RSSI_LEVEL] = \
                self.tahoma_device.active_states['core:RSSILevelState']
        if 'core:SensorDefectState' in self.tahoma_device.active_states:
            attr[ATTR_BATTERY_LEVEL] = \
                self.tahoma_device.active_states['core:SensorDefectState']
        return attr

    @property
    def available(self):
        """Return True if entity is available."""
        return self._available



"""
Support for monitoring pyLoad.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/sensor.pyload/
"""
from datetime import timedelta
import logging

from aiohttp.hdrs import CONTENT_TYPE
import requests
import voluptuous as vol

from homeassistant.components.sensor import PLATFORM_SCHEMA
from homeassistant.const import (
    CONF_SSL, CONF_HOST, CONF_NAME, CONF_PORT, CONF_PASSWORD, CONF_USERNAME,
    CONTENT_TYPE_JSON, CONF_MONITORED_VARIABLES)
import homeassistant.helpers.config_validation as cv
from homeassistant.helpers.entity import Entity
from homeassistant.util import Throttle

_LOGGER = logging.getLogger(__name__)

DEFAULT_HOST = 'localhost'
DEFAULT_NAME = 'pyLoad'
DEFAULT_PORT = 8000

MIN_TIME_BETWEEN_UPDATES = timedelta(seconds=15)

SENSOR_TYPES = {
    'speed': ['speed', 'Speed', 'MB/s'],
}

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Optional(CONF_HOST, default=DEFAULT_HOST): cv.string,
    vol.Optional(CONF_MONITORED_VARIABLES, default=['speed']):
        vol.All(cv.ensure_list, [vol.In(SENSOR_TYPES)]),
    vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
    vol.Optional(CONF_PASSWORD): cv.string,
    vol.Optional(CONF_PORT, default=DEFAULT_PORT): cv.port,
    vol.Optional(CONF_SSL, default=False): cv.boolean,
    vol.Optional(CONF_USERNAME): cv.string,
})


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up the pyLoad sensors."""
    host = config.get(CONF_HOST)
    port = config.get(CONF_PORT)
    ssl = 's' if config.get(CONF_SSL) else ''
    name = config.get(CONF_NAME)
    username = config.get(CONF_USERNAME)
    password = config.get(CONF_PASSWORD)
    monitored_types = config.get(CONF_MONITORED_VARIABLES)
    url = "http{}://{}:{}/api/".format(ssl, host, port)

    try:
        pyloadapi = PyLoadAPI(
            api_url=url, username=username, password=password)
        pyloadapi.update()
    except (requests.exceptions.ConnectionError,
            requests.exceptions.HTTPError) as conn_err:
        _LOGGER.error("Error setting up pyLoad API: %s", conn_err)
        return False

    devices = []
    for ng_type in monitored_types:
        new_sensor = PyLoadSensor(
            api=pyloadapi, sensor_type=SENSOR_TYPES.get(ng_type),
            client_name=name)
        devices.append(new_sensor)

    add_entities(devices, True)


class PyLoadSensor(Entity):
    """Representation of a pyLoad sensor."""

    def __init__(self, api, sensor_type, client_name):
        """Initialize a new pyLoad sensor."""
        self._name = '{} {}'.format(client_name, sensor_type[1])
        self.type = sensor_type[0]
        self.api = api
        self._state = None
        self._unit_of_measurement = sensor_type[2]

    @property
    def name(self):
        """Return the name of the sensor."""
        return self._name

    @property
    def state(self):
        """Return the state of the sensor."""
        return self._state

    @property
    def unit_of_measurement(self):
        """Return the unit of measurement of this entity, if any."""
        return self._unit_of_measurement

    def update(self):
        """Update state of sensor."""
        try:
            self.api.update()
        except requests.exceptions.ConnectionError:
            # Error calling the API, already logged in api.update()
            return

        if self.api.status is None:
            _LOGGER.debug("Update of %s requested, but no status is available",
                          self._name)
            return

        value = self.api.status.get(self.type)
        if value is None:
            _LOGGER.warning("Unable to locate value for %s", self.type)
            return

        if "speed" in self.type and value > 0:
            # Convert download rate from Bytes/s to MBytes/s
            self._state = round(value / 2**20, 2)
        else:
            self._state = value


class PyLoadAPI:
    """Simple wrapper for pyLoad's API."""

    def __init__(self, api_url, username=None, password=None):
        """Initialize pyLoad API and set headers needed later."""
        self.api_url = api_url
        self.status = None
        self.headers = {CONTENT_TYPE: CONTENT_TYPE_JSON}

        if username is not None and password is not None:
            self.payload = {'username': username, 'password': password}
            self.login = requests.post(
                '{}{}'.format(api_url, 'login'), data=self.payload, timeout=5)
        self.update()

    def post(self, method, params=None):
        """Send a POST request and return the response as a dict."""
        payload = {'method': method}

        if params:
            payload['params'] = params

        try:
            response = requests.post(
                '{}{}'.format(self.api_url, 'statusServer'), json=payload,
                cookies=self.login.cookies, headers=self.headers, timeout=5)
            response.raise_for_status()
            _LOGGER.debug("JSON Response: %s", response.json())
            return response.json()

        except requests.exceptions.ConnectionError as conn_exc:
            _LOGGER.error("Failed to update pyLoad status. Error: %s",
                          conn_exc)
            raise

    @Throttle(MIN_TIME_BETWEEN_UPDATES)
    def update(self):
        """Update cached response."""
        self.status = self.post('speed')



"""Component to manage the AIS Cloud."""
import asyncio
import logging
import requests
import json
import os
from homeassistant.ais_dom import ais_global
from homeassistant.const import EVENT_PLATFORM_DISCOVERED, EVENT_STATE_CHANGED
from homeassistant.helpers.discovery import async_load_platform
from homeassistant.const import (CONF_NAME, CONF_IP_ADDRESS, CONF_MAC)
from homeassistant.util import slugify
DOMAIN = 'ais_cloud'
# DEPENDENCIES = ['http']
_LOGGER = logging.getLogger(__name__)
REQUIREMENTS = ['feedparser==5.2.1', 'readability-lxml', 'bs4']
CLOUD_TOKEN_PATH = "/data/data/pl.sviete.dom/files"
CLOUD_TOKEN_PATH += "/home/AIS/.dom/.ais_token.txt"
CLOUD_APP_URL = "https://powiedz.co/ords/f?p=100:1&x01=TOKEN:"
CLOUD_WS_TOKEN = None
CLOUD_WS_HEADER = {}
GLOBAL_RSS_NEWS_TEXT = None
GLOBAL_RSS_HELP_TEXT = None
G_PLAYERS = []


def get_news_text():
    return GLOBAL_RSS_NEWS_TEXT


def get_rss_help_text():
    return GLOBAL_RSS_HELP_TEXT


def check_url(url_address):
    # check the 301 redirection
    try:
        r = requests.head(url_address, allow_redirects=True, timeout=1)
        return r.url
    except:
        return url_address


    # Get player id by his name
def get_player_data(player_name):
    for player in G_PLAYERS:
        if player["friendly_name"] == player_name:
            return player


@asyncio.coroutine
def async_setup(hass, config):
    """Initialize the radio station list."""
    _LOGGER.info("Initialize the radio station list.")
    data = hass.data[DOMAIN] = AisColudData(hass)
    yield from data.get_types_async()

    # add "Projekt" panel to the menu list
    yield from hass.components.frontend.async_register_built_in_panel(
            'iframe', "Projekt", "mdi:shuffle-variant",
            "dom", {'url': 'https://' + ais_global.get_my_global_ip() + ':1880'})

    def get_radio_types(call):
        _LOGGER.info("get_radio_types  ")
        data.get_radio_types(call)

    def get_radio_names(call):
        _LOGGER.info("get_radio_names")
        data.get_radio_names(call)

    def select_radio_name(call):
        _LOGGER.info("select_radio_name")
        data.select_radio_name(call)

    def get_players(call):
        _LOGGER.info("get_players  ")
        data.get_players(call, hass)

    def play_audio(call):
        _LOGGER.info("play_audio  ")
        data.play_audio(call)

    def get_podcast_types(call):
        _LOGGER.info("get_podcast_types  ")
        data.get_podcast_types(call)

    def get_podcast_names(call):
        _LOGGER.info("get_podcast_names  ")
        data.get_podcast_names(call)

    def get_podcast_tracks(call):
        _LOGGER.info("get_podcast_tracks")
        data.get_podcast_tracks(call)

    def select_podcast_track(call):
        _LOGGER.info("select_podcast_track")
        data.select_podcast_track(call)

    def select_media_player(call):
        _LOGGER.info("select_media_player")
        data.select_media_player(call)

    def get_rss_news_channels(call):
        _LOGGER.info("get_rss_news_channels")
        data.get_rss_news_channels(call)

    def get_rss_news_items(call):
        _LOGGER.info("get_rss_news_items  ")
        data.get_rss_news_items(call)

    def select_rss_news_item(call):
        _LOGGER.info("select_rss_news_item")
        data.select_rss_news_item(call)

    def select_rss_help_item(call):
        _LOGGER.info("select_rss_help_item")
        data.select_rss_help_item(call)


    # register services
    hass.services.async_register(
        DOMAIN, 'get_radio_types', get_radio_types)
    hass.services.async_register(
        DOMAIN, 'get_radio_names', get_radio_names)
    hass.services.async_register(
        DOMAIN, 'select_radio_name', select_radio_name)
    hass.services.async_register(
        DOMAIN, 'get_players', get_players)
    hass.services.async_register(
        DOMAIN, 'play_audio', play_audio)
    hass.services.async_register(
        DOMAIN, 'get_podcast_types', get_podcast_types)
    hass.services.async_register(
        DOMAIN, 'get_podcast_names', get_podcast_names)
    hass.services.async_register(
        DOMAIN, 'get_podcast_tracks', get_podcast_tracks)
    hass.services.async_register(
        DOMAIN, 'select_podcast_track', select_podcast_track)
    hass.services.async_register(
        DOMAIN, 'select_media_player', select_media_player)
    hass.services.async_register(
        DOMAIN, 'get_rss_news_channels', get_rss_news_channels)
    hass.services.async_register(
        DOMAIN, 'get_rss_news_items', get_rss_news_items)
    hass.services.async_register(
        DOMAIN, 'select_rss_news_item', select_rss_news_item)
    hass.services.async_register(
        DOMAIN, 'select_rss_help_item', select_rss_help_item)


    def device_discovered(service):
        """ Called when a device has been discovered. """
        _LOGGER.info("Discovered a new device type: " + str(service.as_dict()))
        try:
            d = service.as_dict().get('data')
            s = d.get('service')
            p = d.get('platform')
            if s == 'load_platform.sensor' and p == 'mqtt':
                i = d.get('discovered')
                uid = i.get('unique_id')
                if uid is not None:
                    # search entity_id for this unique_id
                    # add sensor to group
                    hass.async_add_job(
                        hass.services.async_call(
                            'group',
                            'set', {
                                "object_id": "all_ais_sensors",
                                "add_entities": ["sensor." + uid]
                            }
                        )
                    )
            elif s == 'load_platform.media_player':
                hass.async_add_job(
                    hass.services.async_call('ais_cloud', 'get_players')
                )
            # prepare menu
            hass.async_add_job(
                hass.services.async_call(
                'ais_ai_service',
                'prepare_remote_menu'
                )
            )
        except Exception as e:
            _LOGGER.error("device_discovered: " + str(e))

    hass.bus.async_listen(EVENT_PLATFORM_DISCOVERED, device_discovered)


    def state_changed(state_event):
        """ Called on state change """
        entity_id = state_event.data.get('entity_id')
        if entity_id.startswith('media_player.'):
            _new = state_event.data['new_state'].attributes
            if state_event.data['old_state'] is None:
                _old = {}
                _old['friendly_name'] = 'new ais dome device'
            else:
                _old = state_event.data['old_state'].attributes
            # check if name was changed
            if _new['friendly_name'] != _old['friendly_name']:
                hass.async_add_job(
                    hass.services.async_call('ais_cloud', 'get_players')
                )
        elif entity_id == 'input_select.assistant_voice':
            voice = hass.states.get(entity_id).state
            if (voice == 'Kobieta online'):
                ais_global.GLOBAL_TTS_VOICE = 'pl-pl-x-oda-network'
            elif (voice == 'Kobieta lokalnie'):
                ais_global.GLOBAL_TTS_VOICE = 'pl-pl-x-oda-local'
            elif (voice == 'Kobieta 1'):
                ais_global.GLOBAL_TTS_VOICE = 'pl-pl-x-oda#female_1-local'
            elif (voice == 'Kobieta 2'):
                ais_global.GLOBAL_TTS_VOICE = 'pl-pl-x-oda#female_2-local'
            elif (voice == 'Kobieta 3'):
                ais_global.GLOBAL_TTS_VOICE = 'pl-pl-x-oda#female_3-local'
            elif (voice == 'Mężczyzna 1'):
                ais_global.GLOBAL_TTS_VOICE = 'pl-pl-x-oda#male_1-local'
            elif (voice == 'Mężczyzna 2'):
                ais_global.GLOBAL_TTS_VOICE = 'pl-pl-x-oda#male_2-local'
            elif (voice == 'Mężczyzna 3'):
                ais_global.GLOBAL_TTS_VOICE = 'pl-pl-x-oda#male_3-local'
            else:
                ais_global.GLOBAL_TTS_VOICE = 'pl-pl-x-oda-local'
        elif entity_id == 'input_number.assistant_rate':
            try:
                ais_global.GLOBAL_TTS_RATE = float(hass.states.get(entity_id).state)
            except Exception:
                ais_global.GLOBAL_TTS_RATE = 1
        elif entity_id == 'input_number.assistant_tone':
            try:
                ais_global.GLOBAL_TTS_PITCH = float(hass.states.get(entity_id).state)
            except Exception:
                ais_global.GLOBAL_TTS_PITCH = 1

    hass.bus.async_listen(EVENT_STATE_CHANGED, state_changed)
    #
    aisCloud = AisCloudWS()
    CLOUD_WS_TOKEN = aisCloud.getCurrentToken()
    if CLOUD_WS_TOKEN is not None:
        # Show the ais cloud group
        hass.states.async_set(
            'weblink.cloud', CLOUD_APP_URL + CLOUD_WS_TOKEN, {
                'icon': 'mdi:open-in-new',
                'friendly_name': 'Konfiguracja usług'
            })
        yield from hass.services.async_call(
            'group',
            'set_visibility', {
                "entity_id": "group.dom_cloud",
                "visible": True
                })

    else:
        # Hide the group ais cloud group
        yield from hass.services.async_call(
            'group',
            'set_visibility', {
                "entity_id": "group.dom_cloud",
                "visible": False
                })
    return True


class AisCloudWS:
    def __init__(self):
        """Initialize the cloud WS connections."""
        self.url = "https://powiedz.co/ords/dom/dom/"

    def fomatCloudToken(self, token):
        global CLOUD_WS_HEADER, CLOUD_WS_TOKEN
        CLOUD_WS_TOKEN = token
        CLOUD_WS_HEADER = {'Authorization': '{}'.format(CLOUD_WS_TOKEN)}
        _LOGGER.info('CLOUD_WS_HEADER: ' + str(CLOUD_WS_HEADER))

    def getCurrentToken(self):
        # take the token from secrets
        try:
            with open(CLOUD_TOKEN_PATH, 'r') as myfile:
                token = myfile.read().replace('\n', '')
            rest_url = self.url + 'token_info?token=' + token
            ws_resp = requests.get(rest_url, headers=CLOUD_WS_HEADER)
            json_ws_resp = ws_resp.json()
            if (json_ws_resp["valid"] == 1):
                self.fomatCloudToken(token)
                return token
        except Exception as e:
            _LOGGER.error("Exception: " + str(e))
        return None

    def getCloudToken(self, login, password):
        from uuid import getnode as get_mac
        mac = get_mac()
        rest_url = self.url + 'token?mac=' + str(mac)
        try:
            ws_resp = requests.get(rest_url, auth=(login, password))
            json_ws_resp = ws_resp.json()
            token = json_ws_resp["token"]
            with open(CLOUD_TOKEN_PATH, 'w+') as myfile:
                myfile.write(token)
            self.fomatCloudToken(token)
            return token
        except Exception as e:
            _LOGGER.error("Exception: " + str(e))
        return None

    def ask(self, question, org_answer):
        rest_url = self.url + 'ask?question=' + question + " "
        rest_url += '&org_answer=' + org_answer
        _LOGGER.info('rest_url: ' + str(rest_url))
        ws_resp = requests.get(rest_url, headers=CLOUD_WS_HEADER)
        return ws_resp

    def audio_type(self, nature):
        try:
            rest_url = self.url + "audio_type?nature=" + nature
            ws_resp = requests.get(rest_url, headers=CLOUD_WS_HEADER)
            return ws_resp
        except:
            _LOGGER.error("Can't connect to AIS Cloud!!!")
            ais_global.G_OFFLINE_MODE = True

    def audio_name(self, nature, type):
        rest_url = self.url + "audio_name?nature=" + nature
        rest_url += "&type=" + type
        ws_resp = requests.get(rest_url, headers=CLOUD_WS_HEADER)
        return ws_resp

    def audio(self, item, type, text_input):
        rest_url = self.url + "audio?item=" + item + "&type="
        rest_url += type + "&text_input=" + text_input
        ws_resp = requests.get(rest_url, headers=CLOUD_WS_HEADER)
        return ws_resp

    def key(self, service):
        rest_url = self.url + "key?service=" + service
        ws_resp = requests.get(rest_url, headers=CLOUD_WS_HEADER)
        return ws_resp


class AisCacheData:
    def __init__(self, hass):
        """Initialize the files cache"""
        self.hass = hass
        self.persistence_radio = '/dom/radio_stations.json'
        self.persistence_podcast = '/dom/podcast.json'
        self.persistence_news = '/dom/news_chanels.json'

    def get_path(self, nature):
        path = str(os.path.dirname(__file__))
        if nature == ais_global.G_AN_RADIO:
            path = path + self.persistence_radio
        elif nature == ais_global.G_AN_PODCAST:
            path = path + self.persistence_podcast
        elif nature == ais_global.G_AN_NEWS:
            path = path + self.persistence_news
        return path

    def audio_type(self, nature):
        # get types from cache file
        # types = [ais_global.G_EMPTY_OPTION]
        path = self.get_path(nature)
        if not os.path.isfile(path):
            return None
        else:
            with open(path) as file:
                data = json.loads(file.read())
                # items = data["data"]
                # values = set()
                # for item in items:
                #     values.add(item['type'])
                #     types = list(sorted(values))
        return data

    def store_audio_type(self, nature, json_data):
        path = self.get_path(nature)
        with open(path, 'w') as outfile:
            json.dump(json_data, outfile)

    def audio_name(self, nature, type):
        # get names from cache file
        return None
        # names = [ais_global.G_EMPTY_OPTION]
        # path = self.get_path(nature)
        # if not os.path.isfile(path):
        #     return None
        # else:
        #     return names

    def audio(self, item, type, text_input):
        return None


class AisColudData:
    """Class to hold radio stations data."""

    def __init__(self, hass):
        self.hass = hass
        self.radio_names = []
        self.podcast_names = []
        self.podcast_tracks = []
        self.audio_name = None
        self.cloud = AisCloudWS()
        self.cache = AisCacheData(hass)
        self.news_channels = []
        self.news_items = []

    @asyncio.coroutine
    def get_types_async(self):
        def load():
            # check if we have data stored in local files
            # otherwise we should work in online mode and get data from cloud
            # ----------------
            # ----- RADIO ----
            # ----------------
            ws_resp = self.cloud.audio_type(ais_global.G_AN_RADIO)
            if ws_resp is None:
                json_ws_resp = self.cache.audio_type(ais_global.G_AN_RADIO)
            else:
                json_ws_resp = ws_resp.json()
                self.cache.store_audio_type(ais_global.G_AN_RADIO, json_ws_resp)

            types = [ais_global.G_EMPTY_OPTION]
            for item in json_ws_resp["data"]:
                types.append(item)
            # populate list with all stations from selected type
            self.hass.services.call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.radio_type",
                    "options": types})
            # ----------------
            # --- PODCASTS ---
            # ----------------
            ws_resp = self.cloud.audio_type(ais_global.G_AN_PODCAST)
            if ws_resp is None:
                json_ws_resp = self.cache.audio_type(ais_global.G_AN_PODCAST)
            else:
                json_ws_resp = ws_resp.json()
                self.cache.store_audio_type(ais_global.G_AN_PODCAST, json_ws_resp)
            types = [ais_global.G_EMPTY_OPTION]
            for item in json_ws_resp["data"]:
                types.append(item)
            # populate list with all podcast from selected type
            self.hass.services.call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.podcast_type",
                    "options": types})
            # ----------------
            # ----- NEWS -----
            # ----------------
            ws_resp = self.cloud.audio_type(ais_global.G_AN_NEWS)
            if ws_resp is None:
                json_ws_resp = self.cache.audio_type(ais_global.G_AN_NEWS)
            else:
                json_ws_resp = ws_resp.json()
                self.cache.store_audio_type(ais_global.G_AN_NEWS, json_ws_resp)
            types = [ais_global.G_EMPTY_OPTION]
            for item in json_ws_resp["data"]:
                types.append(item)
            # populate list with all news types from selected type
            self.hass.services.call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.rss_news_category",
                    "options": types})
        yield from self.hass.async_add_job(load)

    def get_radio_types(self, call):
        ws_resp = self.cache.audio_type(ais_global.G_AN_RADIO)
        if ws_resp is None:
            ws_resp = self.cloud.audio_type(ais_global.G_AN_RADIO)
        json_ws_resp = ws_resp.json()
        types = [ais_global.G_EMPTY_OPTION]
        for item in json_ws_resp["data"]:
            types.append(item)
        # populate list with all stations from selected type
        self.hass.services.call(
            'input_select',
            'set_options', {
                "entity_id": "input_select.radio_type",
                "options": types})

    def get_radio_names(self, call):
        """Load stations of the for the selected type."""
        if ("radio_type" not in call.data):
            _LOGGER.error("No radio_type")
            return []
        if call.data["radio_type"] == ais_global.G_EMPTY_OPTION:
            # reset status for item below
            self.hass.services.call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.radio_station_name",
                    "options": [ais_global.G_EMPTY_OPTION]})
            return

        ws_resp = self.cache.audio_name(
            ais_global.G_AN_RADIO, call.data["radio_type"])
        if ws_resp is None:
            ws_resp = self.cloud.audio_name(
                ais_global.G_AN_RADIO, call.data["radio_type"])
        json_ws_resp = ws_resp.json()
        self.radio_names = []
        names = [ais_global.G_EMPTY_OPTION]
        for item in json_ws_resp["data"]:
            names.append(item["NAME"])
            self.radio_names.append(item)
        self.hass.services.call(
            'input_select',
            'set_options', {
                "entity_id": "input_select.radio_station_name",
                "options": names})
        # select the radio name
        if self.audio_name is not None:
            self.hass.block_till_done()
            self.hass.services.call(
                'input_select',
                'select_option', {
                    "entity_id": "input_select.radio_station_name",
                    "option": self.audio_name})
            # this name will be set after the list refresh
            self.audio_name = None
        # check if the change was done form remote
        import homeassistant.components.ais_ai_service as ais_ai
        if (ais_ai.CURR_ENTITIE == 'input_select.radio_type'
                and ais_ai.CURR_BUTTON_CODE == 23):
            ais_ai.set_curr_entity(
                self.hass,
                'input_select.radio_station_name')
            self.hass.services.call(
                'ais_ai_service',
                'say_it', {
                    "text": "Wybierz stację"
                })

    def select_radio_name(self, call):
        """Get station stream url for the selected name."""
        if ("radio_name" not in call.data):
            _LOGGER.error("No radio_name")
            return

        # the station was selected from select list in app
        # we need to find the url and play it
        radio_name = call.data["radio_name"]
        _url = None
        _audio_info = {}
        for audio in self.radio_names:
            if audio["NAME"] == radio_name:
                if "STREAM_URL" in audio:
                    _url = check_url(audio["STREAM_URL"])
                    _audio_info["NAME"] = audio["NAME"]
                    _audio_info["MEDIA_SOURCE"] = ais_global.G_AN_RADIO
                    _audio_info["IMAGE_URL"] = audio["IMAGE_URL"]
                    _audio_info = json.dumps(_audio_info)

        if _url is not None:
            # take the entity_id dynamically
            # according to the input_select.radio_player LV
            player_name = self.hass.states.get(
                'input_select.radio_player').state
            player = get_player_data(player_name)
            self.hass.services.call(
                'media_player',
                'play_media', {
                    "entity_id": player["entity_id"],
                    "media_content_type": "audio/mp4",
                    "media_content_id": check_url(_url)
                })
            # set stream image and title only if the player is AIS dom player
            if player["device_ip"] is not None:
                self.hass.services.call(
                    'media_player',
                    'play_media', {
                        "entity_id": player["entity_id"],
                        "media_content_type": "ais_info",
                        "media_content_id": _audio_info
                    })

    def get_podcast_types(self, call):
        ws_resp = self.cloud.audio_type(ais_global.G_AN_PODCAST)
        json_ws_resp = ws_resp.json()
        types = [ais_global.G_EMPTY_OPTION]
        for item in json_ws_resp["data"]:
            types.append(item)
        # populate list with all podcast types
        self.hass.services.call(
            'input_select',
            'set_options', {
                "entity_id": "input_select.podcast_type",
                "options": types})

    def get_podcast_names(self, call):
        """Load podcasts names for the selected type."""
        if ("podcast_type" not in call.data):
            _LOGGER.error("No podcast_type")
            return []
        if call.data["podcast_type"] == ais_global.G_EMPTY_OPTION:
            # reset status for item below
            self.hass.services.call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.podcast_name",
                    "options": [ais_global.G_EMPTY_OPTION]})
            return
        ws_resp = self.cloud.audio_name(
            ais_global.G_AN_PODCAST, call.data["podcast_type"])
        json_ws_resp = ws_resp.json()
        names = [ais_global.G_EMPTY_OPTION]
        self.podcast_names = []
        for item in json_ws_resp["data"]:
            names.append(item["NAME"])
            self.podcast_names.append(item)
        self.hass.services.call(
            'input_select',
            'set_options', {
                "entity_id": "input_select.podcast_name",
                "options": names})
        # check if the change was done form remote
        import homeassistant.components.ais_ai_service as ais_ai
        if (ais_ai.CURR_ENTITIE == 'input_select.podcast_type'
                and ais_ai.CURR_BUTTON_CODE == 23):
            ais_ai.set_curr_entity(
                self.hass,
                'input_select.podcast_name')
            self.hass.services.call(
                'ais_ai_service',
                'say_it', {
                    "text": "Wybierz audycję"
                })

    def get_podcast_tracks(self, call):
        import feedparser
        selected_by_voice_command = False
        if ("podcast_name" not in call.data):
            _LOGGER.error("No podcast_name")
            return
        if call.data["podcast_name"] == ais_global.G_EMPTY_OPTION:
            # reset status for item below
            self.hass.services.call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.podcast_track",
                    "options": [ais_global.G_EMPTY_OPTION]})
            return
        podcast_name = call.data["podcast_name"]
        if ("lookup_url" in call.data):
            _lookup_url = call.data["lookup_url"]
            _image_url = call.data["image_url"]
            selected_by_voice_command = True
        else:
            # the podcast was selected from select list in app
            _lookup_url = None
            _image_url = None
            selected_by_voice_command = False
            for podcast in self.podcast_names:
                if podcast["NAME"] == podcast_name:
                    _lookup_url = podcast["LOOKUP_URL"]
                    _image_url = podcast["IMAGE_URL"]

        if _lookup_url is not None:
            # download the episodes
            self.hass.services.call(
                'ais_ai_service',
                'say_it', {
                    "text": "Pobieram"
                })
            try:
                d = feedparser.parse(check_url(_lookup_url))
                tracks = [ais_global.G_EMPTY_OPTION]
                self.podcast_tracks = []
                for e in d.entries:
                    track = {}
                    track['title'] = e.title
                    track['link'] = e.enclosures[0]
                    try:
                        track['image_url'] = d.feed.image.href
                    except Exception:
                        track['image_url'] = _image_url
                    tracks.append(e.title)
                    self.podcast_tracks.append(track)
                self.hass.services.call(
                    'input_select',
                    'set_options', {
                        "entity_id": "input_select.podcast_track",
                        "options": tracks})

                if (selected_by_voice_command):
                    track = self.podcast_tracks[0]
                    self.hass.services.call(
                        'ais_ai_service',
                        'say_it', {
                            "text": "Pobrano " + str(len(d.entries))
                            + " odcinków"
                            + ", audycji " + podcast_name
                            + ", włączam najnowszy odcinek: " + track["title"]
                        })
                    self.hass.services.call(
                        'input_select',
                        'select_option', {
                            "entity_id": "input_select.podcast_track",
                            "option": track["title"]})
                else:
                    # check if the change was done form remote
                    import homeassistant.components.ais_ai_service as ais_ai
                    if (ais_ai.CURR_ENTITIE
                            == 'input_select.podcast_name'
                            and ais_ai.CURR_BUTTON_CODE == 23):
                            ais_ai.set_curr_entity(
                                self.hass,
                                'input_select.podcast_track')
                            self.hass.services.call(
                                'ais_ai_service',
                                'say_it', {
                                    "text": "Pobrano " + str(len(d.entries))
                                    + " odcinków, wybierz odcinek"
                                })
                    else:
                        self.hass.services.call(
                            'ais_ai_service',
                            'say_it', {
                                "text": "Pobrano " + str(len(d.entries))
                                + " odcinków"
                                + ", audycji " + podcast_name
                            })
            except Exception as e:
                _LOGGER.error("Error: " + str(e))
                self.hass.services.call(
                    'ais_ai_service',
                    'say_it', {
                        "text": "Nie można pobrać odcinków. " + podcast_name
                    })

    def select_podcast_track(self, call):
        """Get track stream url for the selected name."""
        if ("podcast_track" not in call.data):
            _LOGGER.error("No podcast_track")
            return
        if call.data["podcast_track"] == ais_global.G_EMPTY_OPTION:
            # TODO stop selected player
            pass
        # the station was selected from select list in app
        # we need to find the url and play it
        podcast_track = call.data["podcast_track"]
        _url = None
        _audio_info = {}
        for podcast in self.podcast_tracks:
            if podcast["title"] == podcast_track:
                if "link" in podcast:
                    _url = check_url(podcast["link"].href)
                    try:
                        _audio_info["IMAGE_URL"] = podcast["image_url"]
                    except Exception as e:
                        _audio_info["IMAGE_URL"] = ''
                    _audio_info["NAME"] = podcast["title"]
                    _audio_info["MEDIA_SOURCE"] = ais_global.G_AN_PODCAST
                    _audio_info = json.dumps(_audio_info)

        if _url is not None:
            # take the entity_id dynamically
            # according to the input_select.radio_player LV
            player_name = self.hass.states.get(
                'input_select.podcast_player').state
            player = get_player_data(player_name)
            self.hass.services.call(
                'media_player',
                'play_media', {
                    "entity_id": player["entity_id"],
                    "media_content_type": "audio/mp4",
                    "media_content_id": check_url(_url)
                })
            self.hass.services.call(
                'media_player',
                'play_media', {
                    "entity_id": player["entity_id"],
                    "media_content_type": "audio/mp4",
                    "media_content_id": _url
                })
            # set stream image and title
            if player["device_ip"] is not None:
                self.hass.services.call(
                    'media_player',
                    'play_media', {
                        "entity_id": player["entity_id"],
                        "media_content_type": "ais_info",
                        "media_content_id": _audio_info
                    })

    def play_audio(self, call):
        audio_type = call.data["audio_type"]
        if (audio_type == ais_global.G_AN_RADIO):
            self.hass.services.call(
                'input_select',
                'select_option', {
                    "entity_id": "input_select.radio_type",
                    "option": call.data["type"]})
            self.hass.block_till_done()
            self.hass.services.call(
                'input_select',
                'select_option', {
                    "entity_id": "input_select.radio_station_name",
                    "option": call.data["name"]})
            # this name will be set after the list refresh
            self.audio_name = call.data["name"]
            self.hass.block_till_done()
            player_name = self.hass.states.get(
                'input_select.radio_player').state
            player = get_player_data(player_name)
            self.hass.services.call(
                'media_player',
                'play_media', {
                    "entity_id": player["entity_id"],
                    "media_content_type": "audio/mp4",
                    "media_content_id": check_url(call.data["stream_url"])
                })
            # set stream image and title
            if player["device_ip"] is not None:
                _audio_info = {}
                _audio_info["IMAGE_URL"] = call.data["image_url"]
                _audio_info["NAME"] = call.data["name"]
                _audio_info["MEDIA_SOURCE"] = ais_global.G_AN_RADIO
                _audio_info = json.dumps(_audio_info)
                self.hass.services.call(
                    'media_player',
                    'play_media', {
                        "entity_id": player["entity_id"],
                        "media_content_type": "ais_info",
                        "media_content_id": _audio_info
                    })
        if (audio_type == ais_global.G_AN_PODCAST):
            self.hass.services.call(
                'input_select',
                'select_option', {
                    "entity_id": "input_select.podcast_type",
                    "option": call.data["type"]})

            self.hass.services.call(
                'ais_cloud',
                'get_podcast_tracks', {
                    "lookup_url": call.data["lookup_url"],
                    "podcast_name": call.data["name"],
                    "image_url": call.data["image_url"]
                }
            )
            self.hass.services.call(
                'input_select',
                'select_option', {
                    "entity_id": "input_select.podcast_name",
                    "option": call.data["name"]})

        if (audio_type == ais_global.G_AN_MUSIC):
            self.hass.services.call(
                'input_text',
                'set_value', {
                    "entity_id": "input_text.ais_youtube_query",
                    "value": call.data["text"]})

    def select_media_player(self, call):
        if ("media_player_type" not in call.data):
            _LOGGER.error("No media_player_type")
            return
        player_name = None
        _url = None
        _audio_info = {}
        media_player_type = call.data["media_player_type"]
        if media_player_type == "Radio":
            radio_name = self.hass.states.get(
                'input_select.radio_station_name').state
            if radio_name == ais_global.G_EMPTY_OPTION:
                return
            player_name = self.hass.states.get(
                'input_select.radio_player').state
            for radio in self.radio_names:
                if radio["NAME"] == radio_name:
                    if "STREAM_URL" in radio:
                        _url = radio["STREAM_URL"]
                        _audio_info["NAME"] = radio["NAME"]
                        _audio_info["MEDIA_SOURCE"] = ais_global.G_AN_RADIO
                        _audio_info["IMAGE_URL"] = radio["IMAGE_URL"]
        if media_player_type == "Podcast":
            podcast_track = self.hass.states.get(
                'input_select.podcast_track').state
            if podcast_track == ais_global.G_EMPTY_OPTION:
                return
            player_name = self.hass.states.get(
                'input_select.podcast_player').state
            for track in self.podcast_tracks:
                if track["title"] == podcast_track:
                    if "link" in track:
                        _url = track["link"].href
                        try:
                            _audio_info["IMAGE_URL"] = track["image_url"]
                        except Exception as e:
                            _audio_info["IMAGE_URL"] = ''
                        _audio_info["NAME"] = track["title"]
                        _audio_info["MEDIA_SOURCE"] = ais_global.G_AN_PODCAST
        if media_player_type == "Music":
            track_name = self.hass.states.get(
                'input_select.ais_youtube_track_name').state
            if track_name == ais_global.G_EMPTY_OPTION:
                return
            player_name = self.hass.states.get(
                'input_select.ais_music_player').state
            import homeassistant.components.ais_yt_service as yt
            for music_track in yt.G_YT_FOUND:
                if music_track["title"] == track_name:
                    _url = "https://www.youtube.com/watch?v="
                    _url += music_track["id"]
                    _audio_info["IMAGE_URL"] = music_track["thumbnail"]
                    _audio_info["NAME"] = music_track["title"]
                    _audio_info["MEDIA_SOURCE"] = ais_global.G_AN_MUSIC
        if media_player_type == "Book":
            chapter_name = self.hass.states.get(
                'input_select.book_chapter').state
            if chapter_name == ais_global.G_EMPTY_OPTION:
                return
            player_name = self.hass.states.get(
                'input_select.book_player').state
            import homeassistant.components.ais_gm_service as gm
            for ch in gm.G_SELECTED_TRACKS:
                if(ch["name"] == chapter_name):
                    _url = gm.G_GM_MOBILE_CLIENT_API.get_stream_url(ch["id"])
                    _audio_info = {}
                    _audio_info["IMAGE_URL"] = ch["image"]
                    _audio_info["NAME"] = ch["name"]
                    _audio_info["MEDIA_SOURCE"] = ais_global.G_AN_AUDIOBOOK
        if player_name is not None:
            player = get_player_data(player_name)
        if _url is not None:
            # play media on selected device
            self.hass.services.call(
                'media_player',
                'play_media', {
                    "entity_id": player["entity_id"],
                    "media_content_type": "audio/mp4",
                    "media_content_id": check_url(_url)
                })
            if player["device_ip"] is not None:
                # set stream image and title
                self.hass.services.call(
                    'media_player',
                    'play_media', {
                        "entity_id": player["entity_id"],
                        "media_content_type": "ais_info",
                        "media_content_id": json.dumps(_audio_info)
                    })

    def get_players(self, call, hass):
        global G_PLAYERS
        G_PLAYERS = []
        players_lv = []
        if ("device_name" in call.data):
            # check if this device already exists
            name = slugify(call.data.get('device_name'))
            palyer = hass.states.get('media_player.' + name)
            if palyer is None:
                _LOGGER.info("Adding new ais dom player " + name)
                hass.async_run_job(
                    async_load_platform(
                        hass, 'media_player', 'ais_exo_player',
                        {
                            CONF_NAME: call.data.get('device_name'),
                            CONF_IP_ADDRESS: call.data.get(CONF_IP_ADDRESS),
                            CONF_MAC: call.data.get(CONF_MAC)
                        },
                        {}))

        # take the info about normal players
        entities = hass.states.async_all()
        for entity in entities:
            if entity.entity_id.startswith('media_player.'):
                player = {}
                friendly_name = entity.attributes.get('friendly_name')
                device_ip = entity.attributes.get('device_ip')
                player['friendly_name'] = friendly_name
                player['entity_id'] = entity.entity_id
                player['device_ip'] = device_ip
                G_PLAYERS.append(player)
                players_lv.append(friendly_name)
                # add player to group if it's not added
                hass.async_add_job(
                    hass.services.async_call(
                        'group',
                        'set', {
                            "object_id": "audio_player",
                            "add_entities": [entity.entity_id]}))

        hass.async_add_job(
            hass.services.async_call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.radio_player",
                    "options": players_lv}))
        hass.async_add_job(
            hass.services.async_call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.podcast_player",
                    "options": players_lv}))
        hass.async_add_job(
            hass.services.async_call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.ais_music_player",
                    "options": players_lv}))
        hass.async_add_job(
            hass.services.async_call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.book_player",
                    "options": players_lv}))
        hass.async_add_job(
            hass.services.async_call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.rss_news_player",
                    "options": players_lv}))
        # TODO remove Podłączony głośnik from the list
        hass.async_add_job(
            hass.services.async_call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.tts_player",
                    "options": players_lv}))
        hass.async_add_job(
            hass.services.async_call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.file_player",
                    "options": players_lv}))
        # rebuild the groups
        import homeassistant.components.ais_ai_service as ais_ai
        ais_ai.get_groups(hass)


    def get_rss_news_channels(self, call):
        """Load news channels of the for the selected category."""
        if ("rss_news_category" not in call.data):
            _LOGGER.error("No rss_news_category")
            return []
        if call.data["rss_news_category"] == ais_global.G_EMPTY_OPTION:
            # reset status for item below
            self.hass.services.call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.rss_news_channel",
                    "options": [ais_global.G_EMPTY_OPTION]})
            return
        ws_resp = self.cloud.audio_name(
            ais_global.G_AN_NEWS, call.data["rss_news_category"])
        json_ws_resp = ws_resp.json()
        names = [ais_global.G_EMPTY_OPTION]
        self.news_channels = []
        for item in json_ws_resp["data"]:
            names.append(item["NAME"])
            self.news_channels.append(item)
        self.hass.services.call(
            'input_select',
            'set_options', {
                "entity_id": "input_select.rss_news_channel",
                "options": names})
        # check if the change was done form remote
        import homeassistant.components.ais_ai_service as ais_ai
        if (ais_ai.CURR_ENTITIE == 'input_select.rss_news_category'
                and ais_ai.CURR_BUTTON_CODE == 23):
            ais_ai.set_curr_entity(
                self.hass,
                'input_select.rss_news_channel')
            self.hass.services.call(
                'ais_ai_service',
                'say_it', {
                    "text": "Wybierz kanał wiadomości"
                })

    def get_rss_news_items(self, call):
        import feedparser
        selected_by_voice_command = False
        if ("rss_news_channel" not in call.data):
            _LOGGER.error("No rss_news_channel")
            return
        if call.data["rss_news_channel"] == ais_global.G_EMPTY_OPTION:
            # reset status for item below
            self.hass.services.call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.rss_news_item",
                    "options": [ais_global.G_EMPTY_OPTION]})
            return
        rss_news_channel = call.data["rss_news_channel"]
        if ("lookup_url" in call.data):
            _lookup_url = call.data["lookup_url"]
            _image_url = call.data["image_url"]
            selected_by_voice_command = True
        else:
            # the news was selected from select list in app
            _lookup_url = None
            _image_url = None
            selected_by_voice_command = False
            for channel in self.news_channels:
                if channel["NAME"] == rss_news_channel:
                    _lookup_url = channel["LOOKUP_URL"]
                    _image_url = channel["IMAGE_URL"]

        if _lookup_url is not None:
            # download the episodes
            self.hass.services.call(
                'ais_ai_service',
                'say_it', {
                    "text": "pobieram"
                })
            try:
                d = feedparser.parse(_lookup_url)
                items = [ais_global.G_EMPTY_OPTION]
                self.news_items = []
                for e in d.entries:
                    item = {}
                    item['title'] = e.title
                    item['link'] = e.link
                    item['image_url'] = _image_url
                    item['description'] = e.description
                    if e.title not in items:
                        items.append(e.title)
                        self.news_items.append(item)
                self.hass.services.call(
                    'input_select',
                    'set_options', {
                        "entity_id": "input_select.rss_news_item",
                        "options": items})

                if (selected_by_voice_command):
                    item = self.news_items[0]
                    self.hass.services.call(
                        'ais_ai_service',
                        'say_it', {
                            "text": "mamy "
                            + str(len(d.entries)) + " wiadomości z "
                            + rss_news_channel
                            + ", czytam najnowszy artykuł: " + item["title"]
                        })
                    self.hass.services.call(
                        'input_select',
                        'select_option', {
                            "entity_id": "input_select.rss_news_item",
                            "option": item["title"]})

                else:
                    self.hass.services.call(
                        'ais_ai_service',
                        'say_it', {
                            "text": "mamy "
                            + str(len(d.entries))
                            + " wiadomości, wybierz artykuł"
                        })
                    # check if the change was done form remote
                    import homeassistant.components.ais_ai_service as ais_ai
                    if (ais_ai.CURR_ENTITIE
                            == 'input_select.rss_news_channel'
                            and ais_ai.CURR_BUTTON_CODE == 23):
                                ais_ai.set_curr_entity(
                                    self.hass,
                                    'input_select.rss_news_item')

            except Exception as e:
                _LOGGER.error("Error: " + str(e))
                self.hass.services.call(
                    'ais_ai_service',
                    'say_it', {
                        "text": "Nie można pobrać wiadomości z: "
                                + rss_news_channel
                    })

    def select_rss_news_item(self, call):
        """Get text for the selected item."""
        global GLOBAL_RSS_NEWS_TEXT
        if ("rss_news_item" not in call.data):
            _LOGGER.error("No rss_news_item")
            return
        if call.data["rss_news_item"] == ais_global.G_EMPTY_OPTION:
            # reset status for item below
            GLOBAL_RSS_NEWS_TEXT = ''
            self.hass.states.async_set(
                'sensor.rss_news_text', 'ok', {
                    'custom_ui_state_card': 'state-card-text',
                    'text': "" + GLOBAL_RSS_NEWS_TEXT
                    })
            return
        # the station was selected from select list in app
        # we need to find the url and read the text
        rss_news_item = call.data["rss_news_item"]
        _url = None
        for item in self.news_items:
            if item["title"] == rss_news_item:
                if "description" in item:
                    GLOBAL_RSS_NEWS_TEXT = item["description"]
                if "link" in item:
                    _url = check_url(item["link"])

        if _url is not None:
            import requests
            from readability import Document
            response = requests.get(check_url(_url))
            response.encoding = 'utf-8'
            doc = Document(response.text)
            GLOBAL_RSS_NEWS_TEXT += doc.summary()

        from bs4 import BeautifulSoup
        GLOBAL_RSS_NEWS_TEXT = BeautifulSoup(
            GLOBAL_RSS_NEWS_TEXT, "lxml").text

        text = "Czytam artykuł, " + rss_news_item + ". " + GLOBAL_RSS_NEWS_TEXT
        self.hass.services.call(
            'ais_ai_service',
            'say_it', {
                "text": text
            })
        text = "\n\n+" + "Czytam artykuł: \n"
        text += '*' + rss_news_item + "\n" + GLOBAL_RSS_NEWS_TEXT
        GLOBAL_RSS_NEWS_TEXT = text
        self.hass.states.async_set(
            'sensor.rss_news_text', 'ok', {
                'custom_ui_state_card': 'state-card-text',
                'text': "" + GLOBAL_RSS_NEWS_TEXT
                })

    def select_rss_help_item(self, call):
        """Get text for the selected item."""
        global GLOBAL_RSS_HELP_TEXT
        GLOBAL_RSS_HELP_TEXT = ''
        if ("rss_help_topic" not in call.data):
            _LOGGER.error("No rss_help_topic")
            return
        if call.data["rss_help_topic"] == ais_global.G_EMPTY_OPTION:
            # reset status for item below
            self.hass.states.async_set(
                'sensor.ais_rss_help_text', 'ok', {
                    'custom_ui_state_card': 'state-card-text',
                    'text': "" + GLOBAL_RSS_HELP_TEXT
                })
            return
        # we need to build the url and get the text to read
        rss_help_topic = call.data["rss_help_topic"]
        _url = check_url("https://raw.githubusercontent.com/wiki/sviete/AIS-WWW/" + rss_help_topic.replace(" ", "-") + ".md")
        import requests
        from readability import Document
        response = requests.get(_url)
        doc = Document(response.text)
        GLOBAL_RSS_HELP_TEXT += doc.summary()

        from bs4 import BeautifulSoup
        GLOBAL_RSS_HELP_TEXT = BeautifulSoup(
            GLOBAL_RSS_HELP_TEXT, "lxml").text

        text = "Czytam stronę pomocy, " + rss_help_topic + ". " + GLOBAL_RSS_HELP_TEXT
        self.hass.services.call(
            'ais_ai_service',
            'say_it', {
                "text": text
            })
        text = "\n\n+" + "Czytam stronę pomocy: \n"
        text += '*' + rss_help_topic + "\n" + GLOBAL_RSS_HELP_TEXT
        GLOBAL_RSS_HELP_TEXT = text
        self.hass.states.async_set(
            'sensor.ais_rss_help_text', 'ok', {
                'custom_ui_state_card': 'state-card-text',
                'text': "" + GLOBAL_RSS_HELP_TEXT
            })


"""
Support for lights under the iGlo brand.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/light.iglo/
"""
import logging
import math

import voluptuous as vol

from homeassistant.components.light import (
    ATTR_BRIGHTNESS, ATTR_COLOR_TEMP, ATTR_EFFECT, ATTR_HS_COLOR,
    SUPPORT_BRIGHTNESS, SUPPORT_COLOR_TEMP, SUPPORT_COLOR, SUPPORT_EFFECT,
    PLATFORM_SCHEMA, Light)
from homeassistant.const import CONF_HOST, CONF_NAME, CONF_PORT
import homeassistant.helpers.config_validation as cv
import homeassistant.util.color as color_util

REQUIREMENTS = ['iglo==1.2.7']

_LOGGER = logging.getLogger(__name__)

DEFAULT_NAME = 'iGlo Light'
DEFAULT_PORT = 8080

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Required(CONF_HOST): cv.string,
    vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
    vol.Optional(CONF_PORT, default=DEFAULT_PORT): cv.port,
})


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up the iGlo lights."""
    host = config.get(CONF_HOST)
    name = config.get(CONF_NAME)
    port = config.get(CONF_PORT)
    add_entities([IGloLamp(name, host, port)], True)


class IGloLamp(Light):
    """Representation of an iGlo light."""

    def __init__(self, name, host, port):
        """Initialize the light."""
        from iglo import Lamp
        self._name = name
        self._lamp = Lamp(0, host, port)

    @property
    def name(self):
        """Return the name of the light."""
        return self._name

    @property
    def brightness(self):
        """Return the brightness of this light between 0..255."""
        return int((self._lamp.state()['brightness'] / 200.0) * 255)

    @property
    def color_temp(self):
        """Return the color temperature."""
        return color_util.color_temperature_kelvin_to_mired(
            self._lamp.state()['white'])

    @property
    def min_mireds(self):
        """Return the coldest color_temp that this light supports."""
        return math.ceil(color_util.color_temperature_kelvin_to_mired(
            self._lamp.max_kelvin))

    @property
    def max_mireds(self):
        """Return the warmest color_temp that this light supports."""
        return math.ceil(color_util.color_temperature_kelvin_to_mired(
            self._lamp.min_kelvin))

    @property
    def hs_color(self):
        """Return the hs value."""
        return color_util.color_RGB_to_hs(*self._lamp.state()['rgb'])

    @property
    def effect(self):
        """Return the current effect."""
        return self._lamp.state()['effect']

    @property
    def effect_list(self):
        """Return the list of supported effects."""
        return self._lamp.effect_list()

    @property
    def supported_features(self):
        """Flag supported features."""
        return (SUPPORT_BRIGHTNESS | SUPPORT_COLOR_TEMP |
                SUPPORT_COLOR | SUPPORT_EFFECT)

    @property
    def is_on(self):
        """Return true if light is on."""
        return self._lamp.state()['on']

    def turn_on(self, **kwargs):
        """Turn the light on."""
        if not self.is_on:
            self._lamp.switch(True)
        if ATTR_BRIGHTNESS in kwargs:
            brightness = int((kwargs[ATTR_BRIGHTNESS] / 255.0) * 200.0)
            self._lamp.brightness(brightness)
            return

        if ATTR_HS_COLOR in kwargs:
            rgb = color_util.color_hs_to_RGB(*kwargs[ATTR_HS_COLOR])
            self._lamp.rgb(*rgb)
            return

        if ATTR_COLOR_TEMP in kwargs:
            kelvin = int(color_util.color_temperature_mired_to_kelvin(
                kwargs[ATTR_COLOR_TEMP]))
            self._lamp.white(kelvin)
            return

        if ATTR_EFFECT in kwargs:
            effect = kwargs[ATTR_EFFECT]
            self._lamp.effect(effect)
            return

    def turn_off(self, **kwargs):
        """Turn the light off."""
        self._lamp.switch(False)



"""
Show the amount of records in a user's Discogs collection.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/sensor.discogs/
"""
import asyncio
from datetime import timedelta
import logging

import voluptuous as vol

from homeassistant.components.sensor import PLATFORM_SCHEMA
from homeassistant.const import ATTR_ATTRIBUTION, CONF_NAME, CONF_TOKEN
from homeassistant.helpers.aiohttp_client import SERVER_SOFTWARE
import homeassistant.helpers.config_validation as cv
from homeassistant.helpers.entity import Entity

REQUIREMENTS = ['discogs_client==2.2.1']

_LOGGER = logging.getLogger(__name__)

ATTR_IDENTITY = 'identity'

CONF_ATTRIBUTION = "Data provided by Discogs"

DEFAULT_NAME = 'Discogs'

ICON = 'mdi:album'

SCAN_INTERVAL = timedelta(hours=2)

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Required(CONF_TOKEN): cv.string,
    vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
})


@asyncio.coroutine
def async_setup_platform(hass, config, async_add_entities,
                         discovery_info=None):
    """Set up the Discogs sensor."""
    import discogs_client

    name = config.get(CONF_NAME)
    token = config.get(CONF_TOKEN)

    try:
        discogs = discogs_client.Client(SERVER_SOFTWARE, user_token=token)
        identity = discogs.identity()
    except discogs_client.exceptions.HTTPError:
        _LOGGER.error("API token is not valid")
        return

    async_add_entities([DiscogsSensor(identity, name)], True)


class DiscogsSensor(Entity):
    """Get a user's number of records in collection."""

    def __init__(self, identity, name):
        """Initialize the Discogs sensor."""
        self._identity = identity
        self._name = name
        self._state = None

    @property
    def name(self):
        """Return the name of the sensor."""
        return self._name

    @property
    def state(self):
        """Return the state of the sensor."""
        return self._state

    @property
    def icon(self):
        """Return the icon to use in the frontend, if any."""
        return ICON

    @property
    def unit_of_measurement(self):
        """Return the unit this state is expressed in."""
        return 'records'

    @property
    def device_state_attributes(self):
        """Return the state attributes of the sensor."""
        return {
            ATTR_ATTRIBUTION: CONF_ATTRIBUTION,
            ATTR_IDENTITY: self._identity.name,
        }

    @asyncio.coroutine
    def async_update(self):
        """Set state to the amount of records in user's collection."""
        self._state = self._identity.num_collection



"""
Support for GTFS (Google/General Transport Format Schema).

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/sensor.gtfs/
"""
import os
import logging
import datetime
import threading

import voluptuous as vol

from homeassistant.components.sensor import PLATFORM_SCHEMA
from homeassistant.const import CONF_NAME
from homeassistant.helpers.entity import Entity
import homeassistant.helpers.config_validation as cv

REQUIREMENTS = ['pygtfs-homeassistant==0.1.3.dev0']

_LOGGER = logging.getLogger(__name__)

CONF_DATA = 'data'
CONF_DESTINATION = 'destination'
CONF_ORIGIN = 'origin'
CONF_OFFSET = 'offset'

DEFAULT_NAME = 'GTFS Sensor'
DEFAULT_PATH = 'gtfs'

ICON = 'mdi:train'

TIME_FORMAT = '%Y-%m-%d %H:%M:%S'

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend({
    vol.Required(CONF_ORIGIN): cv.string,
    vol.Required(CONF_DESTINATION): cv.string,
    vol.Required(CONF_DATA): cv.string,
    vol.Optional(CONF_NAME): cv.string,
    vol.Optional(CONF_OFFSET, default=0): cv.time_period,
})


def get_next_departure(sched, start_station_id, end_station_id, offset):
    """Get the next departure for the given schedule."""
    origin_station = sched.stops_by_id(start_station_id)[0]
    destination_station = sched.stops_by_id(end_station_id)[0]

    now = datetime.datetime.now() + offset
    day_name = now.strftime('%A').lower()
    now_str = now.strftime('%H:%M:%S')
    today = now.strftime('%Y-%m-%d')

    from sqlalchemy.sql import text

    sql_query = text("""
    SELECT trip.trip_id, trip.route_id,
           time(origin_stop_time.departure_time),
           time(destination_stop_time.arrival_time),
           time(origin_stop_time.arrival_time),
           time(origin_stop_time.departure_time),
           origin_stop_time.drop_off_type,
           origin_stop_time.pickup_type,
           origin_stop_time.shape_dist_traveled,
           origin_stop_time.stop_headsign,
           origin_stop_time.stop_sequence,
           time(destination_stop_time.arrival_time),
           time(destination_stop_time.departure_time),
           destination_stop_time.drop_off_type,
           destination_stop_time.pickup_type,
           destination_stop_time.shape_dist_traveled,
           destination_stop_time.stop_headsign,
           destination_stop_time.stop_sequence
    FROM trips trip
    INNER JOIN calendar calendar
               ON trip.service_id = calendar.service_id
    INNER JOIN stop_times origin_stop_time
               ON trip.trip_id = origin_stop_time.trip_id
    INNER JOIN stops start_station
               ON origin_stop_time.stop_id = start_station.stop_id
    INNER JOIN stop_times destination_stop_time
               ON trip.trip_id = destination_stop_time.trip_id
    INNER JOIN stops end_station
               ON destination_stop_time.stop_id = end_station.stop_id
    WHERE calendar.{day_name} = 1
               AND time(origin_stop_time.departure_time) > time(:now_str)
    AND start_station.stop_id = :origin_station_id
               AND end_station.stop_id = :end_station_id
    AND origin_stop_time.stop_sequence < destination_stop_time.stop_sequence
    AND calendar.start_date <= :today
    AND calendar.end_date >= :today
    ORDER BY origin_stop_time.departure_time LIMIT 1;
    """.format(day_name=day_name))
    result = sched.engine.execute(sql_query, now_str=now_str,
                                  origin_station_id=origin_station.id,
                                  end_station_id=destination_station.id,
                                  today=today)
    item = {}
    for row in result:
        item = row

    if item == {}:
        return None

    departure_time_string = '{} {}'.format(today, item[2])
    arrival_time_string = '{} {}'.format(today, item[3])
    departure_time = datetime.datetime.strptime(departure_time_string,
                                                TIME_FORMAT)
    arrival_time = datetime.datetime.strptime(arrival_time_string,
                                              TIME_FORMAT)

    seconds_until = (departure_time-datetime.datetime.now()).total_seconds()
    minutes_until = int(seconds_until / 60)

    route = sched.routes_by_id(item[1])[0]

    origin_stoptime_arrival_time = '{} {}'.format(today, item[4])
    origin_stoptime_departure_time = '{} {}'.format(today, item[5])
    dest_stoptime_arrival_time = '{} {}'.format(today, item[11])
    dest_stoptime_depart_time = '{} {}'.format(today, item[12])

    origin_stop_time_dict = {
        'Arrival Time': origin_stoptime_arrival_time,
        'Departure Time': origin_stoptime_departure_time,
        'Drop Off Type': item[6], 'Pickup Type': item[7],
        'Shape Dist Traveled': item[8], 'Headsign': item[9],
        'Sequence': item[10]
    }

    destination_stop_time_dict = {
        'Arrival Time': dest_stoptime_arrival_time,
        'Departure Time': dest_stoptime_depart_time,
        'Drop Off Type': item[13], 'Pickup Type': item[14],
        'Shape Dist Traveled': item[15], 'Headsign': item[16],
        'Sequence': item[17]
    }

    return {
        'trip_id': item[0],
        'trip': sched.trips_by_id(item[0])[0],
        'route': route,
        'agency': sched.agencies_by_id(route.agency_id)[0],
        'origin_station': origin_station,
        'departure_time': departure_time,
        'destination_station': destination_station,
        'arrival_time': arrival_time,
        'seconds_until_departure': seconds_until,
        'minutes_until_departure': minutes_until,
        'origin_stop_time': origin_stop_time_dict,
        'destination_stop_time': destination_stop_time_dict
    }


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up the GTFS sensor."""
    gtfs_dir = hass.config.path(DEFAULT_PATH)
    data = config.get(CONF_DATA)
    origin = config.get(CONF_ORIGIN)
    destination = config.get(CONF_DESTINATION)
    name = config.get(CONF_NAME)
    offset = config.get(CONF_OFFSET)

    if not os.path.exists(gtfs_dir):
        os.makedirs(gtfs_dir)

    if not os.path.exists(os.path.join(gtfs_dir, data)):
        _LOGGER.error("The given GTFS data file/folder was not found")
        return False

    import pygtfs

    split_file_name = os.path.splitext(data)

    sqlite_file = "{}.sqlite".format(split_file_name[0])
    joined_path = os.path.join(gtfs_dir, sqlite_file)
    gtfs = pygtfs.Schedule(joined_path)

    # pylint: disable=no-member
    if not gtfs.feeds:
        pygtfs.append_feed(gtfs, os.path.join(gtfs_dir, data))

    add_entities([
        GTFSDepartureSensor(gtfs, name, origin, destination, offset)])


class GTFSDepartureSensor(Entity):
    """Implementation of an GTFS departures sensor."""

    def __init__(self, pygtfs, name, origin, destination, offset):
        """Initialize the sensor."""
        self._pygtfs = pygtfs
        self.origin = origin
        self.destination = destination
        self._offset = offset
        self._custom_name = name
        self._name = ''
        self._unit_of_measurement = 'min'
        self._state = 0
        self._attributes = {}
        self.lock = threading.Lock()
        self.update()

    @property
    def name(self):
        """Return the name of the sensor."""
        return self._name

    @property
    def state(self):
        """Return the state of the sensor."""
        return self._state

    @property
    def unit_of_measurement(self):
        """Return the unit of measurement of this entity, if any."""
        return self._unit_of_measurement

    @property
    def device_state_attributes(self):
        """Return the state attributes."""
        return self._attributes

    @property
    def icon(self):
        """Icon to use in the frontend, if any."""
        return ICON

    def update(self):
        """Get the latest data from GTFS and update the states."""
        with self.lock:
            self._departure = get_next_departure(
                self._pygtfs, self.origin, self.destination, self._offset)
            if not self._departure:
                self._state = 0
                self._attributes = {'Info': 'No more departures today'}
                if self._name == '':
                    self._name = (self._custom_name or DEFAULT_NAME)
                return

            self._state = self._departure['minutes_until_departure']

            origin_station = self._departure['origin_station']
            destination_station = self._departure['destination_station']
            origin_stop_time = self._departure['origin_stop_time']
            destination_stop_time = self._departure['destination_stop_time']
            agency = self._departure['agency']
            route = self._departure['route']
            trip = self._departure['trip']

            name = '{} {} to {} next departure'
            self._name = (self._custom_name or
                          name.format(agency.agency_name,
                                      origin_station.stop_id,
                                      destination_station.stop_id))

            # Build attributes
            self._attributes = {}
            self._attributes['offset'] = self._offset.seconds / 60

            def dict_for_table(resource):
                """Return a dict for the SQLAlchemy resource given."""
                return dict((col, getattr(resource, col))
                            for col in resource.__table__.columns.keys())

            def append_keys(resource, prefix=None):
                """Properly format key val pairs to append to attributes."""
                for key, val in resource.items():
                    if val == "" or val is None or key == 'feed_id':
                        continue
                    pretty_key = key.replace('_', ' ')
                    pretty_key = pretty_key.title()
                    pretty_key = pretty_key.replace('Id', 'ID')
                    pretty_key = pretty_key.replace('Url', 'URL')
                    if prefix is not None and \
                       pretty_key.startswith(prefix) is False:
                        pretty_key = '{} {}'.format(prefix, pretty_key)
                    self._attributes[pretty_key] = val

            append_keys(dict_for_table(agency), 'Agency')
            append_keys(dict_for_table(route), 'Route')
            append_keys(dict_for_table(trip), 'Trip')
            append_keys(dict_for_table(origin_station), 'Origin Station')
            append_keys(dict_for_table(destination_station),
                        'Destination Station')
            append_keys(origin_stop_time, 'Origin Stop')
            append_keys(destination_stop_time, 'Destination Stop')



"""
Support for interfacing with Monoprice Blackbird 4k 8x8 HDBaseT Matrix.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/media_player.blackbird
"""
import logging
import socket

import voluptuous as vol

from homeassistant.components.media_player import (
    DOMAIN, MEDIA_PLAYER_SCHEMA, PLATFORM_SCHEMA, SUPPORT_SELECT_SOURCE,
    SUPPORT_TURN_OFF, SUPPORT_TURN_ON, MediaPlayerDevice)
from homeassistant.const import (
    ATTR_ENTITY_ID, CONF_HOST, CONF_NAME, CONF_PORT, CONF_TYPE, STATE_OFF,
    STATE_ON)
import homeassistant.helpers.config_validation as cv

REQUIREMENTS = ['pyblackbird==0.5']

_LOGGER = logging.getLogger(__name__)

SUPPORT_BLACKBIRD = SUPPORT_TURN_ON | SUPPORT_TURN_OFF | SUPPORT_SELECT_SOURCE

ZONE_SCHEMA = vol.Schema({
    vol.Required(CONF_NAME): cv.string,
})

SOURCE_SCHEMA = vol.Schema({
    vol.Required(CONF_NAME): cv.string,
})

CONF_ZONES = 'zones'
CONF_SOURCES = 'sources'

DATA_BLACKBIRD = 'blackbird'

SERVICE_SETALLZONES = 'blackbird_set_all_zones'
ATTR_SOURCE = 'source'

BLACKBIRD_SETALLZONES_SCHEMA = MEDIA_PLAYER_SCHEMA.extend({
    vol.Required(ATTR_SOURCE): cv.string
})


# Valid zone ids: 1-8
ZONE_IDS = vol.All(vol.Coerce(int), vol.Range(min=1, max=8))

# Valid source ids: 1-8
SOURCE_IDS = vol.All(vol.Coerce(int), vol.Range(min=1, max=8))

PLATFORM_SCHEMA = vol.All(
    cv.has_at_least_one_key(CONF_PORT, CONF_HOST),
    PLATFORM_SCHEMA.extend({
        vol.Exclusive(CONF_PORT, CONF_TYPE): cv.string,
        vol.Exclusive(CONF_HOST, CONF_TYPE): cv.string,
        vol.Required(CONF_ZONES): vol.Schema({ZONE_IDS: ZONE_SCHEMA}),
        vol.Required(CONF_SOURCES): vol.Schema({SOURCE_IDS: SOURCE_SCHEMA}),
    }))


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up the Monoprice Blackbird 4k 8x8 HDBaseT Matrix platform."""
    if DATA_BLACKBIRD not in hass.data:
        hass.data[DATA_BLACKBIRD] = {}

    port = config.get(CONF_PORT)
    host = config.get(CONF_HOST)

    from pyblackbird import get_blackbird
    from serial import SerialException

    connection = None
    if port is not None:
        try:
            blackbird = get_blackbird(port)
            connection = port
        except SerialException:
            _LOGGER.error("Error connecting to the Blackbird controller")
            return

    if host is not None:
        try:
            blackbird = get_blackbird(host, False)
            connection = host
        except socket.timeout:
            _LOGGER.error("Error connecting to the Blackbird controller")
            return

    sources = {source_id: extra[CONF_NAME] for source_id, extra
               in config[CONF_SOURCES].items()}

    devices = []
    for zone_id, extra in config[CONF_ZONES].items():
        _LOGGER.info("Adding zone %d - %s", zone_id, extra[CONF_NAME])
        unique_id = "{}-{}".format(connection, zone_id)
        device = BlackbirdZone(blackbird, sources, zone_id, extra[CONF_NAME])
        hass.data[DATA_BLACKBIRD][unique_id] = device
        devices.append(device)

    add_entities(devices, True)

    def service_handle(service):
        """Handle for services."""
        entity_ids = service.data.get(ATTR_ENTITY_ID)
        source = service.data.get(ATTR_SOURCE)
        if entity_ids:
            devices = [device for device in hass.data[DATA_BLACKBIRD].values()
                       if device.entity_id in entity_ids]

        else:
            devices = hass.data[DATA_BLACKBIRD].values()

        for device in devices:
            if service.service == SERVICE_SETALLZONES:
                device.set_all_zones(source)

    hass.services.register(DOMAIN, SERVICE_SETALLZONES, service_handle,
                           schema=BLACKBIRD_SETALLZONES_SCHEMA)


class BlackbirdZone(MediaPlayerDevice):
    """Representation of a Blackbird matrix zone."""

    def __init__(self, blackbird, sources, zone_id, zone_name):
        """Initialize new zone."""
        self._blackbird = blackbird
        # dict source_id -> source name
        self._source_id_name = sources
        # dict source name -> source_id
        self._source_name_id = {v: k for k, v in sources.items()}
        # ordered list of all source names
        self._source_names = sorted(self._source_name_id.keys(),
                                    key=lambda v: self._source_name_id[v])
        self._zone_id = zone_id
        self._name = zone_name
        self._state = None
        self._source = None

    def update(self):
        """Retrieve latest state."""
        state = self._blackbird.zone_status(self._zone_id)
        if not state:
            return
        self._state = STATE_ON if state.power else STATE_OFF
        idx = state.av
        if idx in self._source_id_name:
            self._source = self._source_id_name[idx]
        else:
            self._source = None

    @property
    def name(self):
        """Return the name of the zone."""
        return self._name

    @property
    def state(self):
        """Return the state of the zone."""
        return self._state

    @property
    def supported_features(self):
        """Return flag of media commands that are supported."""
        return SUPPORT_BLACKBIRD

    @property
    def media_title(self):
        """Return the current source as media title."""
        return self._source

    @property
    def source(self):
        """Return the current input source of the device."""
        return self._source

    @property
    def source_list(self):
        """List of available input sources."""
        return self._source_names

    def set_all_zones(self, source):
        """Set all zones to one source."""
        if source not in self._source_name_id:
            return
        idx = self._source_name_id[source]
        _LOGGER.debug("Setting all zones source to %s", idx)
        self._blackbird.set_all_zone_source(idx)

    def select_source(self, source):
        """Set input source."""
        if source not in self._source_name_id:
            return
        idx = self._source_name_id[source]
        _LOGGER.debug("Setting zone %d source to %s", self._zone_id, idx)
        self._blackbird.set_zone_source(self._zone_id, idx)

    def turn_on(self):
        """Turn the media player on."""
        _LOGGER.debug("Turning zone %d on", self._zone_id)
        self._blackbird.set_zone_power(self._zone_id, True)

    def turn_off(self):
        """Turn the media player off."""
        _LOGGER.debug("Turning zone %d off", self._zone_id)
        self._blackbird.set_zone_power(self._zone_id, False)



"""
This component provides basic support for Abode Home Security system.

For more details about this component, please refer to the documentation at
https://home-assistant.io/components/abode/
"""
import asyncio
import logging
from functools import partial
from requests.exceptions import HTTPError, ConnectTimeout

import voluptuous as vol

from homeassistant.const import (
    ATTR_ATTRIBUTION, ATTR_DATE, ATTR_TIME, ATTR_ENTITY_ID, CONF_USERNAME,
    CONF_PASSWORD, CONF_EXCLUDE, CONF_NAME, CONF_LIGHTS,
    EVENT_HOMEASSISTANT_STOP, EVENT_HOMEASSISTANT_START)
from homeassistant.helpers import config_validation as cv
from homeassistant.helpers import discovery
from homeassistant.helpers.entity import Entity

REQUIREMENTS = ['abodepy==0.13.1']

_LOGGER = logging.getLogger(__name__)

CONF_ATTRIBUTION = "Data provided by goabode.com"
CONF_POLLING = 'polling'

DOMAIN = 'abode'
DEFAULT_CACHEDB = './abodepy_cache.pickle'

NOTIFICATION_ID = 'abode_notification'
NOTIFICATION_TITLE = 'Abode Security Setup'

EVENT_ABODE_ALARM = 'abode_alarm'
EVENT_ABODE_ALARM_END = 'abode_alarm_end'
EVENT_ABODE_AUTOMATION = 'abode_automation'
EVENT_ABODE_FAULT = 'abode_panel_fault'
EVENT_ABODE_RESTORE = 'abode_panel_restore'

SERVICE_SETTINGS = 'change_setting'
SERVICE_CAPTURE_IMAGE = 'capture_image'
SERVICE_TRIGGER = 'trigger_quick_action'

ATTR_DEVICE_ID = 'device_id'
ATTR_DEVICE_NAME = 'device_name'
ATTR_DEVICE_TYPE = 'device_type'
ATTR_EVENT_CODE = 'event_code'
ATTR_EVENT_NAME = 'event_name'
ATTR_EVENT_TYPE = 'event_type'
ATTR_EVENT_UTC = 'event_utc'
ATTR_SETTING = 'setting'
ATTR_USER_NAME = 'user_name'
ATTR_VALUE = 'value'

ABODE_DEVICE_ID_LIST_SCHEMA = vol.Schema([str])

CONFIG_SCHEMA = vol.Schema({
    DOMAIN: vol.Schema({
        vol.Required(CONF_USERNAME): cv.string,
        vol.Required(CONF_PASSWORD): cv.string,
        vol.Optional(CONF_NAME): cv.string,
        vol.Optional(CONF_POLLING, default=False): cv.boolean,
        vol.Optional(CONF_EXCLUDE, default=[]): ABODE_DEVICE_ID_LIST_SCHEMA,
        vol.Optional(CONF_LIGHTS, default=[]): ABODE_DEVICE_ID_LIST_SCHEMA
    }),
}, extra=vol.ALLOW_EXTRA)

CHANGE_SETTING_SCHEMA = vol.Schema({
    vol.Required(ATTR_SETTING): cv.string,
    vol.Required(ATTR_VALUE): cv.string
})

CAPTURE_IMAGE_SCHEMA = vol.Schema({
    ATTR_ENTITY_ID: cv.entity_ids,
})

TRIGGER_SCHEMA = vol.Schema({
    ATTR_ENTITY_ID: cv.entity_ids,
})

ABODE_PLATFORMS = [
    'alarm_control_panel', 'binary_sensor', 'lock', 'switch', 'cover',
    'camera', 'light', 'sensor'
]


class AbodeSystem:
    """Abode System class."""

    def __init__(self, username, password, cache,
                 name, polling, exclude, lights):
        """Initialize the system."""
        import abodepy
        self.abode = abodepy.Abode(
            username, password, auto_login=True, get_devices=True,
            get_automations=True, cache_path=cache)
        self.name = name
        self.polling = polling
        self.exclude = exclude
        self.lights = lights
        self.devices = []

    def is_excluded(self, device):
        """Check if a device is configured to be excluded."""
        return device.device_id in self.exclude

    def is_automation_excluded(self, automation):
        """Check if an automation is configured to be excluded."""
        return automation.automation_id in self.exclude

    def is_light(self, device):
        """Check if a switch device is configured as a light."""
        import abodepy.helpers.constants as CONST

        return (device.generic_type == CONST.TYPE_LIGHT or
                (device.generic_type == CONST.TYPE_SWITCH and
                 device.device_id in self.lights))


def setup(hass, config):
    """Set up Abode component."""
    from abodepy.exceptions import AbodeException

    conf = config[DOMAIN]
    username = conf.get(CONF_USERNAME)
    password = conf.get(CONF_PASSWORD)
    name = conf.get(CONF_NAME)
    polling = conf.get(CONF_POLLING)
    exclude = conf.get(CONF_EXCLUDE)
    lights = conf.get(CONF_LIGHTS)

    try:
        cache = hass.config.path(DEFAULT_CACHEDB)
        hass.data[DOMAIN] = AbodeSystem(
            username, password, cache, name, polling, exclude, lights)
    except (AbodeException, ConnectTimeout, HTTPError) as ex:
        _LOGGER.error("Unable to connect to Abode: %s", str(ex))

        hass.components.persistent_notification.create(
            'Error: {}<br />'
            'You will need to restart hass after fixing.'
            ''.format(ex),
            title=NOTIFICATION_TITLE,
            notification_id=NOTIFICATION_ID)
        return False

    setup_hass_services(hass)
    setup_hass_events(hass)
    setup_abode_events(hass)

    for platform in ABODE_PLATFORMS:
        discovery.load_platform(hass, platform, DOMAIN, {}, config)

    return True


def setup_hass_services(hass):
    """Home assistant services."""
    from abodepy.exceptions import AbodeException

    def change_setting(call):
        """Change an Abode system setting."""
        setting = call.data.get(ATTR_SETTING)
        value = call.data.get(ATTR_VALUE)

        try:
            hass.data[DOMAIN].abode.set_setting(setting, value)
        except AbodeException as ex:
            _LOGGER.warning(ex)

    def capture_image(call):
        """Capture a new image."""
        entity_ids = call.data.get(ATTR_ENTITY_ID)

        target_devices = [device for device in hass.data[DOMAIN].devices
                          if device.entity_id in entity_ids]

        for device in target_devices:
            device.capture()

    def trigger_quick_action(call):
        """Trigger a quick action."""
        entity_ids = call.data.get(ATTR_ENTITY_ID, None)

        target_devices = [device for device in hass.data[DOMAIN].devices
                          if device.entity_id in entity_ids]

        for device in target_devices:
            device.trigger()

    hass.services.register(
        DOMAIN, SERVICE_SETTINGS, change_setting,
        schema=CHANGE_SETTING_SCHEMA)

    hass.services.register(
        DOMAIN, SERVICE_CAPTURE_IMAGE, capture_image,
        schema=CAPTURE_IMAGE_SCHEMA)

    hass.services.register(
        DOMAIN, SERVICE_TRIGGER, trigger_quick_action,
        schema=TRIGGER_SCHEMA)


def setup_hass_events(hass):
    """Home Assistant start and stop callbacks."""
    def startup(event):
        """Listen for push events."""
        hass.data[DOMAIN].abode.events.start()

    def logout(event):
        """Logout of Abode."""
        if not hass.data[DOMAIN].polling:
            hass.data[DOMAIN].abode.events.stop()

        hass.data[DOMAIN].abode.logout()
        _LOGGER.info("Logged out of Abode")

    if not hass.data[DOMAIN].polling:
        hass.bus.listen_once(EVENT_HOMEASSISTANT_START, startup)

    hass.bus.listen_once(EVENT_HOMEASSISTANT_STOP, logout)


def setup_abode_events(hass):
    """Event callbacks."""
    import abodepy.helpers.timeline as TIMELINE

    def event_callback(event, event_json):
        """Handle an event callback from Abode."""
        data = {
            ATTR_DEVICE_ID: event_json.get(ATTR_DEVICE_ID, ''),
            ATTR_DEVICE_NAME: event_json.get(ATTR_DEVICE_NAME, ''),
            ATTR_DEVICE_TYPE: event_json.get(ATTR_DEVICE_TYPE, ''),
            ATTR_EVENT_CODE: event_json.get(ATTR_EVENT_CODE, ''),
            ATTR_EVENT_NAME: event_json.get(ATTR_EVENT_NAME, ''),
            ATTR_EVENT_TYPE: event_json.get(ATTR_EVENT_TYPE, ''),
            ATTR_EVENT_UTC: event_json.get(ATTR_EVENT_UTC, ''),
            ATTR_USER_NAME: event_json.get(ATTR_USER_NAME, ''),
            ATTR_DATE: event_json.get(ATTR_DATE, ''),
            ATTR_TIME: event_json.get(ATTR_TIME, ''),
        }

        hass.bus.fire(event, data)

    events = [TIMELINE.ALARM_GROUP, TIMELINE.ALARM_END_GROUP,
              TIMELINE.PANEL_FAULT_GROUP, TIMELINE.PANEL_RESTORE_GROUP,
              TIMELINE.AUTOMATION_GROUP]

    for event in events:
        hass.data[DOMAIN].abode.events.add_event_callback(
            event,
            partial(event_callback, event))


class AbodeDevice(Entity):
    """Representation of an Abode device."""

    def __init__(self, data, device):
        """Initialize a sensor for Abode device."""
        self._data = data
        self._device = device

    @asyncio.coroutine
    def async_added_to_hass(self):
        """Subscribe Abode events."""
        self.hass.async_add_job(
            self._data.abode.events.add_device_callback,
            self._device.device_id, self._update_callback
        )

    @property
    def should_poll(self):
        """Return the polling state."""
        return self._data.polling

    def update(self):
        """Update automation state."""
        self._device.refresh()

    @property
    def name(self):
        """Return the name of the sensor."""
        return self._device.name

    @property
    def device_state_attributes(self):
        """Return the state attributes."""
        return {
            ATTR_ATTRIBUTION: CONF_ATTRIBUTION,
            'device_id': self._device.device_id,
            'battery_low': self._device.battery_low,
            'no_response': self._device.no_response,
            'device_type': self._device.type
        }

    def _update_callback(self, device):
        """Update the device state."""
        self.schedule_update_ha_state()


class AbodeAutomation(Entity):
    """Representation of an Abode automation."""

    def __init__(self, data, automation, event=None):
        """Initialize for Abode automation."""
        self._data = data
        self._automation = automation
        self._event = event

    @asyncio.coroutine
    def async_added_to_hass(self):
        """Subscribe Abode events."""
        if self._event:
            self.hass.async_add_job(
                self._data.abode.events.add_event_callback,
                self._event, self._update_callback
            )

    @property
    def should_poll(self):
        """Return the polling state."""
        return self._data.polling

    def update(self):
        """Update automation state."""
        self._automation.refresh()

    @property
    def name(self):
        """Return the name of the sensor."""
        return self._automation.name

    @property
    def device_state_attributes(self):
        """Return the state attributes."""
        return {
            ATTR_ATTRIBUTION: CONF_ATTRIBUTION,
            'automation_id': self._automation.automation_id,
            'type': self._automation.type,
            'sub_type': self._automation.sub_type
        }

    def _update_callback(self, device):
        """Update the device state."""
        self._automation.refresh()
        self.schedule_update_ha_state()



"""Handle MySensors gateways."""
import asyncio
from collections import defaultdict
import logging
import socket
import sys
from timeit import default_timer as timer

import async_timeout
import voluptuous as vol

from homeassistant.const import (
    CONF_NAME, CONF_OPTIMISTIC, EVENT_HOMEASSISTANT_STOP)
from homeassistant.core import callback
from homeassistant.helpers import discovery
import homeassistant.helpers.config_validation as cv
from homeassistant.helpers.dispatcher import async_dispatcher_send
from homeassistant.setup import async_setup_component

from .const import (
    ATTR_DEVICES, CONF_BAUD_RATE, CONF_DEVICE, CONF_GATEWAYS, CONF_NODES,
    CONF_PERSISTENCE, CONF_PERSISTENCE_FILE, CONF_RETAIN, CONF_TCP_PORT,
    CONF_TOPIC_IN_PREFIX, CONF_TOPIC_OUT_PREFIX, CONF_VERSION, DOMAIN,
    MYSENSORS_CONST_SCHEMA, MYSENSORS_GATEWAYS, PLATFORM, SCHEMA,
    SIGNAL_CALLBACK, TYPE)
from .device import get_mysensors_devices

_LOGGER = logging.getLogger(__name__)

GATEWAY_READY_TIMEOUT = 15.0
MQTT_COMPONENT = 'mqtt'
MYSENSORS_GATEWAY_READY = 'mysensors_gateway_ready_{}'


def is_serial_port(value):
    """Validate that value is a windows serial port or a unix device."""
    if sys.platform.startswith('win'):
        ports = ('COM{}'.format(idx + 1) for idx in range(256))
        if value in ports:
            return value
        raise vol.Invalid('{} is not a serial port'.format(value))
    return cv.isdevice(value)


def is_socket_address(value):
    """Validate that value is a valid address."""
    try:
        socket.getaddrinfo(value, None)
        return value
    except OSError:
        raise vol.Invalid('Device is not a valid domain name or ip address')


def get_mysensors_gateway(hass, gateway_id):
    """Return MySensors gateway."""
    if MYSENSORS_GATEWAYS not in hass.data:
        hass.data[MYSENSORS_GATEWAYS] = {}
    gateways = hass.data.get(MYSENSORS_GATEWAYS)
    return gateways.get(gateway_id)


async def setup_gateways(hass, config):
    """Set up all gateways."""
    conf = config[DOMAIN]
    gateways = {}

    for index, gateway_conf in enumerate(conf[CONF_GATEWAYS]):
        persistence_file = gateway_conf.get(
            CONF_PERSISTENCE_FILE,
            hass.config.path('mysensors{}.pickle'.format(index + 1)))
        ready_gateway = await _get_gateway(
            hass, config, gateway_conf, persistence_file)
        if ready_gateway is not None:
            gateways[id(ready_gateway)] = ready_gateway

    return gateways


async def _get_gateway(hass, config, gateway_conf, persistence_file):
    """Return gateway after setup of the gateway."""
    from mysensors import mysensors

    conf = config[DOMAIN]
    persistence = conf[CONF_PERSISTENCE]
    version = conf[CONF_VERSION]
    device = gateway_conf[CONF_DEVICE]
    baud_rate = gateway_conf[CONF_BAUD_RATE]
    tcp_port = gateway_conf[CONF_TCP_PORT]
    in_prefix = gateway_conf.get(CONF_TOPIC_IN_PREFIX, '')
    out_prefix = gateway_conf.get(CONF_TOPIC_OUT_PREFIX, '')

    if device == MQTT_COMPONENT:
        if not await async_setup_component(hass, MQTT_COMPONENT, config):
            return None
        mqtt = hass.components.mqtt
        retain = conf[CONF_RETAIN]

        def pub_callback(topic, payload, qos, retain):
            """Call MQTT publish function."""
            mqtt.async_publish(topic, payload, qos, retain)

        def sub_callback(topic, sub_cb, qos):
            """Call MQTT subscribe function."""
            @callback
            def internal_callback(*args):
                """Call callback."""
                sub_cb(*args)

            hass.async_add_job(
                mqtt.async_subscribe(topic, internal_callback, qos))

        gateway = mysensors.AsyncMQTTGateway(
            pub_callback, sub_callback, in_prefix=in_prefix,
            out_prefix=out_prefix, retain=retain, loop=hass.loop,
            event_callback=None, persistence=persistence,
            persistence_file=persistence_file,
            protocol_version=version)
    else:
        try:
            await hass.async_add_job(is_serial_port, device)
            gateway = mysensors.AsyncSerialGateway(
                device, baud=baud_rate, loop=hass.loop,
                event_callback=None, persistence=persistence,
                persistence_file=persistence_file,
                protocol_version=version)
        except vol.Invalid:
            try:
                await hass.async_add_job(is_socket_address, device)
                # valid ip address
                gateway = mysensors.AsyncTCPGateway(
                    device, port=tcp_port, loop=hass.loop, event_callback=None,
                    persistence=persistence, persistence_file=persistence_file,
                    protocol_version=version)
            except vol.Invalid:
                # invalid ip address
                return None
    gateway.metric = hass.config.units.is_metric
    gateway.optimistic = conf[CONF_OPTIMISTIC]
    gateway.device = device
    gateway.event_callback = _gw_callback_factory(hass)
    gateway.nodes_config = gateway_conf[CONF_NODES]
    if persistence:
        await gateway.start_persistence()

    return gateway


async def finish_setup(hass, gateways):
    """Load any persistent devices and platforms and start gateway."""
    discover_tasks = []
    start_tasks = []
    for gateway in gateways.values():
        discover_tasks.append(_discover_persistent_devices(hass, gateway))
        start_tasks.append(_gw_start(hass, gateway))
    if discover_tasks:
        # Make sure all devices and platforms are loaded before gateway start.
        await asyncio.wait(discover_tasks, loop=hass.loop)
    if start_tasks:
        await asyncio.wait(start_tasks, loop=hass.loop)


async def _discover_persistent_devices(hass, gateway):
    """Discover platforms for devices loaded via persistence file."""
    tasks = []
    new_devices = defaultdict(list)
    for node_id in gateway.sensors:
        node = gateway.sensors[node_id]
        for child in node.children.values():
            validated = _validate_child(gateway, node_id, child)
            for platform, dev_ids in validated.items():
                new_devices[platform].extend(dev_ids)
    for platform, dev_ids in new_devices.items():
        tasks.append(_discover_mysensors_platform(hass, platform, dev_ids))
    if tasks:
        await asyncio.wait(tasks, loop=hass.loop)


@callback
def _discover_mysensors_platform(hass, platform, new_devices):
    """Discover a MySensors platform."""
    task = hass.async_create_task(discovery.async_load_platform(
        hass, platform, DOMAIN,
        {ATTR_DEVICES: new_devices, CONF_NAME: DOMAIN}))
    return task


async def _gw_start(hass, gateway):
    """Start the gateway."""
    # Don't use hass.async_create_task to avoid holding up setup indefinitely.
    connect_task = hass.loop.create_task(gateway.start())

    @callback
    def gw_stop(event):
        """Trigger to stop the gateway."""
        hass.async_add_job(gateway.stop())
        if not connect_task.done():
            connect_task.cancel()

    hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, gw_stop)
    if gateway.device == 'mqtt':
        # Gatways connected via mqtt doesn't send gateway ready message.
        return
    gateway_ready = asyncio.Future()
    gateway_ready_key = MYSENSORS_GATEWAY_READY.format(id(gateway))
    hass.data[gateway_ready_key] = gateway_ready

    try:
        with async_timeout.timeout(GATEWAY_READY_TIMEOUT, loop=hass.loop):
            await gateway_ready
    except asyncio.TimeoutError:
        _LOGGER.warning(
            "Gateway %s not ready after %s secs so continuing with setup",
            gateway.device, GATEWAY_READY_TIMEOUT)
    finally:
        hass.data.pop(gateway_ready_key, None)


def _gw_callback_factory(hass):
    """Return a new callback for the gateway."""
    @callback
    def mysensors_callback(msg):
        """Handle messages from a MySensors gateway."""
        start = timer()
        _LOGGER.debug(
            "Node update: node %s child %s", msg.node_id, msg.child_id)

        _set_gateway_ready(hass, msg)

        try:
            child = msg.gateway.sensors[msg.node_id].children[msg.child_id]
        except KeyError:
            _LOGGER.debug("Not a child update for node %s", msg.node_id)
            return

        signals = []

        # Update all platforms for the device via dispatcher.
        # Add/update entity if schema validates to true.
        validated = _validate_child(msg.gateway, msg.node_id, child)
        for platform, dev_ids in validated.items():
            devices = get_mysensors_devices(hass, platform)
            new_dev_ids = []
            for dev_id in dev_ids:
                if dev_id in devices:
                    signals.append(SIGNAL_CALLBACK.format(*dev_id))
                else:
                    new_dev_ids.append(dev_id)
            if new_dev_ids:
                _discover_mysensors_platform(hass, platform, new_dev_ids)
        for signal in set(signals):
            # Only one signal per device is needed.
            # A device can have multiple platforms, ie multiple schemas.
            # FOR LATER: Add timer to not signal if another update comes in.
            async_dispatcher_send(hass, signal)
        end = timer()
        if end - start > 0.1:
            _LOGGER.debug(
                "Callback for node %s child %s took %.3f seconds",
                msg.node_id, msg.child_id, end - start)
    return mysensors_callback


@callback
def _set_gateway_ready(hass, msg):
    """Set asyncio future result if gateway is ready."""
    if (msg.type != msg.gateway.const.MessageType.internal or
            msg.sub_type != msg.gateway.const.Internal.I_GATEWAY_READY):
        return
    gateway_ready = hass.data.get(MYSENSORS_GATEWAY_READY.format(
        id(msg.gateway)))
    if gateway_ready is None or gateway_ready.cancelled():
        return
    gateway_ready.set_result(True)


def _validate_child(gateway, node_id, child):
    """Validate that a child has the correct values according to schema.

    Return a dict of platform with a list of device ids for validated devices.
    """
    validated = defaultdict(list)

    if not child.values:
        _LOGGER.debug(
            "No child values for node %s child %s", node_id, child.id)
        return validated
    if gateway.sensors[node_id].sketch_name is None:
        _LOGGER.debug("Node %s is missing sketch name", node_id)
        return validated
    pres = gateway.const.Presentation
    set_req = gateway.const.SetReq
    s_name = next(
        (member.name for member in pres if member.value == child.type), None)
    if s_name not in MYSENSORS_CONST_SCHEMA:
        _LOGGER.warning("Child type %s is not supported", s_name)
        return validated
    child_schemas = MYSENSORS_CONST_SCHEMA[s_name]

    def msg(name):
        """Return a message for an invalid schema."""
        return "{} requires value_type {}".format(
            pres(child.type).name, set_req[name].name)

    for schema in child_schemas:
        platform = schema[PLATFORM]
        v_name = schema[TYPE]
        value_type = next(
            (member.value for member in set_req if member.name == v_name),
            None)
        if value_type is None:
            continue
        _child_schema = child.get_schema(gateway.protocol_version)
        vol_schema = _child_schema.extend(
            {vol.Required(set_req[key].value, msg=msg(key)):
             _child_schema.schema.get(set_req[key].value, val)
             for key, val in schema.get(SCHEMA, {v_name: cv.string}).items()},
            extra=vol.ALLOW_EXTRA)
        try:
            vol_schema(child.values)
        except vol.Invalid as exc:
            level = (logging.WARNING if value_type in child.values
                     else logging.DEBUG)
            _LOGGER.log(
                level,
                "Invalid values: %s: %s platform: node %s child %s: %s",
                child.values, platform, node_id, child.id, exc)
            continue
        dev_id = id(gateway), node_id, child.id, value_type
        validated[platform].append(dev_id)
    return validated



"""
Component that will help guide the user taking its first steps.

"""
import asyncio
import logging

import voluptuous as vol

DOMAIN = 'introduction'

CONFIG_SCHEMA = vol.Schema({
    DOMAIN: vol.Schema({}),
}, extra=vol.ALLOW_EXTRA)


@asyncio.coroutine
def async_setup(hass, config=None):
    """Set up the introduction component."""
    log = logging.getLogger(__name__)
    log.info("""

    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        Cześć, witamy w systemie Asystent domowy!

        Mamy nadzieję, że spełnimy wszystkie twoje marzenia.

        Oto kilka informacji, od których możesz zacząć:

         - Konfiguracja urządzeń :
           http://ai-speaker.com

         - Ustawiania audio :
           http://ai-speaker.com

         - Źródła programu:
           https://github.com/sviete

         - Odpowiedzi na najczęściej zadawane pytania:
           https://github.com/sviete


    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    """)

    hass.components.persistent_notification.async_create("""

Mamy nadzieję, że spełnimy wszystkie twoje marzenia.

Oto kilka informacji, od których możesz zacząć:

 - [Pierwsze kroki](https://github.com/sviete/AIS-WWW/wiki/1.-Pierwsze-kroki)
 - [Komendy głosowe](https://github.com/sviete/AIS-WWW/wiki/3.-Komendy-g%C5%82osowe)
 - [Zdalny dostęp](https://github.com/sviete/AIS-WWW/wiki/7.-Zdalny-dost%C4%99p)
 - [Źródła systemu](https://github.com/sviete)
 - [Strona projektu](https://ai-speaker.com)

""", 'Cześć, witamy w systemie Asystent domowy!')  # noqa

    return True



"""Helpers for data entry flows for config entries."""
from functools import partial

from homeassistant.core import callback
from homeassistant import config_entries, data_entry_flow


def register_discovery_flow(domain, title, discovery_function):
    """Register flow for discovered integrations that not require auth."""
    config_entries.HANDLERS.register(domain)(
        partial(DiscoveryFlowHandler, domain, title, discovery_function))


class DiscoveryFlowHandler(data_entry_flow.FlowHandler):
    """Handle a discovery config flow."""

    VERSION = 1

    def __init__(self, domain, title, discovery_function):
        """Initialize the discovery config flow."""
        self._domain = domain
        self._title = title
        self._discovery_function = discovery_function

    async def async_step_user(self, user_input=None):
        """Handle a flow initialized by the user."""
        if self._async_current_entries():
            return self.async_abort(
                reason='single_instance_allowed'
            )

        # Get current discovered entries.
        in_progress = self._async_in_progress()

        has_devices = in_progress
        if not has_devices:
            has_devices = await self.hass.async_add_job(
                self._discovery_function, self.hass)

        if not has_devices:
            return self.async_abort(
                reason='no_devices_found'
            )

        # Cancel the discovered one.
        for flow in in_progress:
            self.hass.config_entries.flow.async_abort(flow['flow_id'])

        return self.async_create_entry(
            title=self._title,
            data={},
        )

    async def async_step_confirm(self, user_input=None):
        """Confirm setup."""
        if user_input is not None:
            return self.async_create_entry(
                title=self._title,
                data={},
            )

        return self.async_show_form(
            step_id='confirm',
        )

    async def async_step_discovery(self, discovery_info):
        """Handle a flow initialized by discovery."""
        if self._async_in_progress() or self._async_current_entries():
            return self.async_abort(
                reason='single_instance_allowed'
            )

        return await self.async_step_confirm()

    async def async_step_import(self, _):
        """Handle a flow initialized by import."""
        if self._async_in_progress() or self._async_current_entries():
            return self.async_abort(
                reason='single_instance_allowed'
            )

        return self.async_create_entry(
            title=self._title,
            data={},
        )

    @callback
    def _async_current_entries(self):
        """Return current entries."""
        return self.hass.config_entries.async_entries(self._domain)

    @callback
    def _async_in_progress(self):
        """Return other in progress flows for current domain."""
        return [flw for flw in self.hass.config_entries.flow.async_progress()
                if flw['handler'] == self._domain and
                flw['flow_id'] != self.flow_id]



"""
Support for MQTT cover devices.

For more details about this platform, please refer to the documentation at
https://home-assistant.io/components/cover.mqtt/
"""
import logging

import voluptuous as vol

from homeassistant.core import callback
from homeassistant.components import mqtt
from homeassistant.components.cover import (
    CoverDevice, ATTR_TILT_POSITION, SUPPORT_OPEN_TILT,
    SUPPORT_CLOSE_TILT, SUPPORT_STOP_TILT, SUPPORT_SET_TILT_POSITION,
    SUPPORT_OPEN, SUPPORT_CLOSE, SUPPORT_STOP, SUPPORT_SET_POSITION,
    ATTR_POSITION)
from homeassistant.exceptions import TemplateError
from homeassistant.const import (
    CONF_NAME, CONF_VALUE_TEMPLATE, CONF_OPTIMISTIC, STATE_OPEN,
    STATE_CLOSED, STATE_UNKNOWN)
from homeassistant.components.mqtt import (
    CONF_AVAILABILITY_TOPIC, CONF_STATE_TOPIC, CONF_COMMAND_TOPIC,
    CONF_PAYLOAD_AVAILABLE, CONF_PAYLOAD_NOT_AVAILABLE, CONF_QOS, CONF_RETAIN,
    valid_publish_topic, valid_subscribe_topic, MqttAvailability)
import homeassistant.helpers.config_validation as cv

_LOGGER = logging.getLogger(__name__)

DEPENDENCIES = ['mqtt']

CONF_TILT_COMMAND_TOPIC = 'tilt_command_topic'
CONF_TILT_STATUS_TOPIC = 'tilt_status_topic'
CONF_POSITION_TOPIC = 'set_position_topic'
CONF_SET_POSITION_TEMPLATE = 'set_position_template'

CONF_PAYLOAD_OPEN = 'payload_open'
CONF_PAYLOAD_CLOSE = 'payload_close'
CONF_PAYLOAD_STOP = 'payload_stop'
CONF_STATE_OPEN = 'state_open'
CONF_STATE_CLOSED = 'state_closed'
CONF_TILT_CLOSED_POSITION = 'tilt_closed_value'
CONF_TILT_OPEN_POSITION = 'tilt_opened_value'
CONF_TILT_MIN = 'tilt_min'
CONF_TILT_MAX = 'tilt_max'
CONF_TILT_STATE_OPTIMISTIC = 'tilt_optimistic'
CONF_TILT_INVERT_STATE = 'tilt_invert_state'

DEFAULT_NAME = 'MQTT Cover'
DEFAULT_PAYLOAD_OPEN = 'OPEN'
DEFAULT_PAYLOAD_CLOSE = 'CLOSE'
DEFAULT_PAYLOAD_STOP = 'STOP'
DEFAULT_OPTIMISTIC = False
DEFAULT_RETAIN = False
DEFAULT_TILT_CLOSED_POSITION = 0
DEFAULT_TILT_OPEN_POSITION = 100
DEFAULT_TILT_MIN = 0
DEFAULT_TILT_MAX = 100
DEFAULT_TILT_OPTIMISTIC = False
DEFAULT_TILT_INVERT_STATE = False

OPEN_CLOSE_FEATURES = (SUPPORT_OPEN | SUPPORT_CLOSE | SUPPORT_STOP)
TILT_FEATURES = (SUPPORT_OPEN_TILT | SUPPORT_CLOSE_TILT | SUPPORT_STOP_TILT |
                 SUPPORT_SET_TILT_POSITION)

PLATFORM_SCHEMA = mqtt.MQTT_BASE_PLATFORM_SCHEMA.extend({
    vol.Optional(CONF_COMMAND_TOPIC): valid_publish_topic,
    vol.Optional(CONF_POSITION_TOPIC): valid_publish_topic,
    vol.Optional(CONF_SET_POSITION_TEMPLATE): cv.template,
    vol.Optional(CONF_RETAIN, default=DEFAULT_RETAIN): cv.boolean,
    vol.Optional(CONF_STATE_TOPIC): valid_subscribe_topic,
    vol.Optional(CONF_VALUE_TEMPLATE): cv.template,
    vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
    vol.Optional(CONF_PAYLOAD_OPEN, default=DEFAULT_PAYLOAD_OPEN): cv.string,
    vol.Optional(CONF_PAYLOAD_CLOSE, default=DEFAULT_PAYLOAD_CLOSE): cv.string,
    vol.Optional(CONF_PAYLOAD_STOP, default=DEFAULT_PAYLOAD_STOP): cv.string,
    vol.Optional(CONF_STATE_OPEN, default=STATE_OPEN): cv.string,
    vol.Optional(CONF_STATE_CLOSED, default=STATE_CLOSED): cv.string,
    vol.Optional(CONF_OPTIMISTIC, default=DEFAULT_OPTIMISTIC): cv.boolean,
    vol.Optional(CONF_TILT_COMMAND_TOPIC): valid_publish_topic,
    vol.Optional(CONF_TILT_STATUS_TOPIC): valid_subscribe_topic,
    vol.Optional(CONF_TILT_CLOSED_POSITION,
                 default=DEFAULT_TILT_CLOSED_POSITION): int,
    vol.Optional(CONF_TILT_OPEN_POSITION,
                 default=DEFAULT_TILT_OPEN_POSITION): int,
    vol.Optional(CONF_TILT_MIN, default=DEFAULT_TILT_MIN): int,
    vol.Optional(CONF_TILT_MAX, default=DEFAULT_TILT_MAX): int,
    vol.Optional(CONF_TILT_STATE_OPTIMISTIC,
                 default=DEFAULT_TILT_OPTIMISTIC): cv.boolean,
    vol.Optional(CONF_TILT_INVERT_STATE,
                 default=DEFAULT_TILT_INVERT_STATE): cv.boolean,
}).extend(mqtt.MQTT_AVAILABILITY_SCHEMA.schema)


async def async_setup_platform(hass, config, async_add_entities,
                               discovery_info=None):
    """Set up the MQTT Cover."""
    if discovery_info is not None:
        config = PLATFORM_SCHEMA(discovery_info)

    value_template = config.get(CONF_VALUE_TEMPLATE)
    if value_template is not None:
        value_template.hass = hass
    set_position_template = config.get(CONF_SET_POSITION_TEMPLATE)
    if set_position_template is not None:
        set_position_template.hass = hass

    async_add_entities([MqttCover(
        config.get(CONF_NAME),
        config.get(CONF_STATE_TOPIC),
        config.get(CONF_COMMAND_TOPIC),
        config.get(CONF_AVAILABILITY_TOPIC),
        config.get(CONF_TILT_COMMAND_TOPIC),
        config.get(CONF_TILT_STATUS_TOPIC),
        config.get(CONF_QOS),
        config.get(CONF_RETAIN),
        config.get(CONF_STATE_OPEN),
        config.get(CONF_STATE_CLOSED),
        config.get(CONF_PAYLOAD_OPEN),
        config.get(CONF_PAYLOAD_CLOSE),
        config.get(CONF_PAYLOAD_STOP),
        config.get(CONF_PAYLOAD_AVAILABLE),
        config.get(CONF_PAYLOAD_NOT_AVAILABLE),
        config.get(CONF_OPTIMISTIC),
        value_template,
        config.get(CONF_TILT_OPEN_POSITION),
        config.get(CONF_TILT_CLOSED_POSITION),
        config.get(CONF_TILT_MIN),
        config.get(CONF_TILT_MAX),
        config.get(CONF_TILT_STATE_OPTIMISTIC),
        config.get(CONF_TILT_INVERT_STATE),
        config.get(CONF_POSITION_TOPIC),
        set_position_template,
    )])


class MqttCover(MqttAvailability, CoverDevice):
    """Representation of a cover that can be controlled using MQTT."""

    def __init__(self, name, state_topic, command_topic, availability_topic,
                 tilt_command_topic, tilt_status_topic, qos, retain,
                 state_open, state_closed, payload_open, payload_close,
                 payload_stop, payload_available, payload_not_available,
                 optimistic, value_template, tilt_open_position,
                 tilt_closed_position, tilt_min, tilt_max, tilt_optimistic,
                 tilt_invert, position_topic, set_position_template):
        """Initialize the cover."""
        super().__init__(availability_topic, qos, payload_available,
                         payload_not_available)
        self._position = None
        self._state = None
        self._name = name
        self._state_topic = state_topic
        self._command_topic = command_topic
        self._tilt_command_topic = tilt_command_topic
        self._tilt_status_topic = tilt_status_topic
        self._qos = qos
        self._payload_open = payload_open
        self._payload_close = payload_close
        self._payload_stop = payload_stop
        self._state_open = state_open
        self._state_closed = state_closed
        self._retain = retain
        self._tilt_open_position = tilt_open_position
        self._tilt_closed_position = tilt_closed_position
        self._optimistic = optimistic or state_topic is None
        self._template = value_template
        self._tilt_value = None
        self._tilt_min = tilt_min
        self._tilt_max = tilt_max
        self._tilt_optimistic = tilt_optimistic
        self._tilt_invert = tilt_invert
        self._position_topic = position_topic
        self._set_position_template = set_position_template

    async def async_added_to_hass(self):
        """Subscribe MQTT events."""
        await super().async_added_to_hass()

        @callback
        def tilt_updated(topic, payload, qos):
            """Handle tilt updates."""
            if (payload.isnumeric() and
                    self._tilt_min <= int(payload) <= self._tilt_max):

                level = self.find_percentage_in_range(float(payload))
                self._tilt_value = level
                self.async_schedule_update_ha_state()

        @callback
        def state_message_received(topic, payload, qos):
            """Handle new MQTT state messages."""
            if self._template is not None:
                payload = self._template.async_render_with_possible_json_value(
                    payload)

            if payload == self._state_open:
                self._state = False
            elif payload == self._state_closed:
                self._state = True
            elif payload.isnumeric() and 0 <= int(payload) <= 100:
                if int(payload) > 0:
                    self._state = False
                else:
                    self._state = True
                self._position = int(payload)
            else:
                _LOGGER.warning(
                    "Payload is not True, False, or integer (0-100): %s",
                    payload)
                return

            self.async_schedule_update_ha_state()

        if self._state_topic is None:
            # Force into optimistic mode.
            self._optimistic = True
        else:
            await mqtt.async_subscribe(
                self.hass, self._state_topic,
                state_message_received, self._qos)

        if self._tilt_status_topic is None:
            self._tilt_optimistic = True
        else:
            self._tilt_optimistic = False
            self._tilt_value = STATE_UNKNOWN
            await mqtt.async_subscribe(
                self.hass, self._tilt_status_topic, tilt_updated, self._qos)

    @property
    def should_poll(self):
        """No polling needed."""
        return False

    @property
    def assumed_state(self):
        """Return true if we do optimistic updates."""
        return self._optimistic

    @property
    def name(self):
        """Return the name of the cover."""
        return self._name

    @property
    def is_closed(self):
        """Return if the cover is closed."""
        return self._state

    @property
    def current_cover_position(self):
        """Return current position of cover.

        None is unknown, 0 is closed, 100 is fully open.
        """
        return self._position

    @property
    def current_cover_tilt_position(self):
        """Return current position of cover tilt."""
        return self._tilt_value

    @property
    def supported_features(self):
        """Flag supported features."""
        supported_features = 0
        if self._command_topic is not None:
            supported_features = OPEN_CLOSE_FEATURES

        if self._position_topic is not None:
            supported_features |= SUPPORT_SET_POSITION

        if self._tilt_command_topic is not None:
            supported_features |= TILT_FEATURES

        return supported_features

    async def async_open_cover(self, **kwargs):
        """Move the cover up.

        This method is a coroutine.
        """
        mqtt.async_publish(
            self.hass, self._command_topic, self._payload_open, self._qos,
            self._retain)
        if self._optimistic:
            # Optimistically assume that cover has changed state.
            self._state = False
            self.async_schedule_update_ha_state()

    async def async_close_cover(self, **kwargs):
        """Move the cover down.

        This method is a coroutine.
        """
        mqtt.async_publish(
            self.hass, self._command_topic, self._payload_close, self._qos,
            self._retain)
        if self._optimistic:
            # Optimistically assume that cover has changed state.
            self._state = True
            self.async_schedule_update_ha_state()

    async def async_stop_cover(self, **kwargs):
        """Stop the device.

        This method is a coroutine.
        """
        mqtt.async_publish(
            self.hass, self._command_topic, self._payload_stop, self._qos,
            self._retain)

    async def async_open_cover_tilt(self, **kwargs):
        """Tilt the cover open."""
        mqtt.async_publish(self.hass, self._tilt_command_topic,
                           self._tilt_open_position, self._qos,
                           self._retain)
        if self._tilt_optimistic:
            self._tilt_value = self._tilt_open_position
            self.async_schedule_update_ha_state()

    async def async_close_cover_tilt(self, **kwargs):
        """Tilt the cover closed."""
        mqtt.async_publish(self.hass, self._tilt_command_topic,
                           self._tilt_closed_position, self._qos,
                           self._retain)
        if self._tilt_optimistic:
            self._tilt_value = self._tilt_closed_position
            self.async_schedule_update_ha_state()

    async def async_set_cover_tilt_position(self, **kwargs):
        """Move the cover tilt to a specific position."""
        if ATTR_TILT_POSITION not in kwargs:
            return

        position = float(kwargs[ATTR_TILT_POSITION])

        # The position needs to be between min and max
        level = self.find_in_range_from_percent(position)

        mqtt.async_publish(self.hass, self._tilt_command_topic,
                           level, self._qos, self._retain)

    async def async_set_cover_position(self, **kwargs):
        """Move the cover to a specific position."""
        if ATTR_POSITION in kwargs:
            position = kwargs[ATTR_POSITION]
            if self._set_position_template is not None:
                try:
                    position = self._set_position_template.async_render(
                        **kwargs)
                except TemplateError as ex:
                    _LOGGER.error(ex)
                    self._state = None

            mqtt.async_publish(self.hass, self._position_topic,
                               position, self._qos, self._retain)

    def find_percentage_in_range(self, position):
        """Find the 0-100% value within the specified range."""
        # the range of motion as defined by the min max values
        tilt_range = self._tilt_max - self._tilt_min
        # offset to be zero based
        offset_position = position - self._tilt_min
        # the percentage value within the range
        position_percentage = float(offset_position) / tilt_range * 100.0
        if self._tilt_invert:
            return 100 - position_percentage
        return position_percentage

    def find_in_range_from_percent(self, percentage):
        """
        Find the adjusted value for 0-100% within the specified range.

        if the range is 80-180 and the percentage is 90
        this method would determine the value to send on the topic
        by offsetting the max and min, getting the percentage value and
        returning the offset
        """
        offset = self._tilt_min
        tilt_range = self._tilt_max - self._tilt_min

        position = round(tilt_range * (percentage / 100.0))
        position += offset

        if self._tilt_invert:
            position = self._tilt_max - position + offset
        return position
